################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
Variants.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, re, optparse, collections

import Experiment as E
import IOTools
import Genomics
import NCL as ncl

Variant = collections.namedtuple( "Variant", 
                                  "pos, reference, genotype" ) 
ExtendedVariant = collections.namedtuple( "Variant", 
                                  "start, end, reference, action, has_wildtype, variantseqs" )

def updateVariants( variants, lcontig, strand, phased = True ):
    '''update variants such that they use same coordinate
    system (and strand) as the transcript

    fixes 1-ness of variants
    '''

    new_variants = []
    is_positive = Genomics.IsPositiveStrand( strand )
    
    for variant in variants:

        pos = variant.pos
        genotype = bytes(variant.genotype)
        reference = bytes(variant.reference)

        # fix 1-ness of variants
        # pos -= 1

        if len(genotype) == 1:
            variantseqs = list(Genomics.decodeGenotype( genotype ) )
            has_wildtype = reference in variantseqs
            action = "="
            start, end = pos, pos+1
        else:

            variantseqs = [ x[1:] for x in genotype.split("/") ]
            lvariant = max( [len(x) for x in variantseqs ] )
            if not phased: variantseqs = [ x for x in variantseqs if x ]
            has_wildtype = "*" in genotype

            if "+" in genotype and "-" in genotype:
                # both insertion and deletion at position
                # the range is given by the deletion
                # see below for explanations
                if genotype.startswith("+"):
                    action = ">"
                    variantseqs[1] += "-" * (lvariant - len(variantseqs[1]))
                else:
                    action = "<"
                    variantseqs[0] += "-" * (lvariant - len(variantseqs[0]))

                start, end = pos + 1, pos + lvariant + 1

            elif "-" in genotype:
                action = "-"
                # samtools: deletions are after the base denoted by snp.position
                #   * <- deletion at 1
                # 0 1 2 3 4 5 6
                #     - -
                # 6 5 4 3 2 1 0
                # deletion of 2+3 = (2,4)
                # on reverse: (7-4, 7-2) = (3,5)
                start, end = pos + 1, pos + lvariant + 1

                # deletions of unequal length are filled up with "-"
                # This is necessary to deal with negative strands:
                # -at/-atg on the positive strand deletes a t [g]
                # -at/-atg on the negative strand deletes [g] t a
                variantseqs = [ x + "-" * (lvariant - len( x )) for x in variantseqs ]

            elif "+" in genotype:
                action = "+"
                # indels are after the base denoted by position
                # as region use both flanking base so that negative strand
                # coordinates work
                # insertion between position 2 and 3
                #     * <- insection at pos 2
                # 0 1 2i3 4 
                # 4 3 2i1 0
                # is insertion between 1 and 2 in reverse
                # including both flanking residues makes it work:
                # (2,3) = (5-3,5-2) = (2,3) 
                # but:
                # (2,4) = (5-4,5-2) = (1,3)  
                start, end = pos, pos + 2

        # revert strand
        if not is_positive: 
            reference = Genomics.complement( reference )
            variantseqs = [ Genomics.complement( x.upper() ) for x in variantseqs ]
            start, end = lcontig - end, lcontig - start

        new_variants.append( ExtendedVariant._make( (
                    start, end, reference.upper(), action, has_wildtype, variantseqs ) ))

    return new_variants

def mergeVariants( variants ):
    '''merge overlapping variants.

    Overlapping variants occur if there are two deletions
    at the same location:

        WT      ACTG  
        Allele1 -CT-   
        Allele2 ----
        
    This will be encoded by samtools as (0-based coordinates)::
        
        0 * -A/ACTG
        3 * -G/-G
     
    This upsets the re-constitution algoritm.

    This method separates these two variants into two non-overlapping
    variants making use of variable length deletions.

        0 * -A/-A
        1 * ---G/-CTG

    Another case:
        
        WT      ACTG  
        Allele1 ACT-   
        Allele2 ----

    This will be encoded by samtools as (0-based coordinates)::
        
        0 * */-ACTG
        3 * -G/*

    This method separates these two as::
        
        0 * */-ACT
        3 * -G/-G

    '''
    
    if len(variants) == 0: return []

    # sorts by start and then end
    variants.sort()
    merged_variants = []

    def _add( offset, dest, src ):
        for x, c in enumerate(src): dest[x+offset] = c

    def _split( seq0, seq1 ):
        # split 
        was_0, was_1 = seq0[0] == "-", seq1[0] == "-"
        for x, cc in enumerate( zip( seq0,seq1 ) ):            
            is_0, is_1 = cc[0] == "-", cc[1] == "-"
            # yield all changes
            if (is_0 ^ was_0) or (is_1 ^ was_1):
                yield x, was_0, was_1
            was_0, was_1 = is_0, is_1

        yield x+1, was_0, was_1

    last = variants[0]
    for this in variants[1:]:

        if this.start < last.end and \
                this.action == "-" and \
                last.action == "-":

            E.warn( "merging overlapping deletions: %s and %s" % (str(last), str(this)) )

            mend = max( last.end, this.end)
            mstart = min( this.start, last.start )
            l = mend - mstart

            seq0 = list("-" * l)            
            seq1 = list("-" * l)

            _add(last.start - mstart, seq0, last.variantseqs[0] )
            _add(last.start - mstart, seq1, last.variantseqs[1] )
            _add(this.start - mstart, seq0, this.variantseqs[0] )
            _add(this.start - mstart, seq1, this.variantseqs[1] )

            last_x = 0
            n = []
            for x, was_0, was_1 in _split( seq0, seq1 ):
                if last_x == x: continue

                this = ExtendedVariant._make( (mstart + last_x,
                                               mstart + x,
                                               "*",
                                               last.action,
                                               was_0 ^ was_1,
                                               [ "".join(seq0[last_x:x]),
                                                 "".join(seq1[last_x:x]) ],
                                               ) )
                n.append( this )
                last_x = x
                
            E.warn( "overlapping deletions merged in %i blocks as: %s" % \
                        (len(n), map(str, n )))
            merged_variants.extend( n[:-1] )
            this = n[-1]
        else:
            merged_variants.append( last )
           
        last = this

    merged_variants.append( last )

    return merged_variants
    
    

def indexVariants( variants ):
    '''build index of variants for ranged retrieval.'''
    index = ncl.NCL()

    for start, end, reference, action, has_wildtype, variantseqs in variants:
        index.add( start, end, (reference,action,has_wildtype, variantseqs) )
        
    return index

def buildAlleles( sequence, variants, reference_start = 0, phased = True):
    '''build alleles for ``sequence`` adding ``variants``.
    
    Variants are assumed to be in 0-based coordinates on the same strand as the sequence. 
    ``reference_start`` is the position of the first base of ``sequence``. Set to 0, if
    the positions in ``variants`` are relative to ``sequence``.
    '''

    def _delete( allele, del_start, del_end, variant, sequence, startoffset, endoffset, feature_start, feature_end ):
        '''little helper: update ``allele`` with a deletion ``del_start:del_end``.
        '''

        # truncate variant according to the feature
        variant = variant[startoffset:len(variant)-endoffset]

        n = variant.count("-")
        if n:
            if variant.startswith("-"):
                del_start += n
                variant = variant[n:]
            else:
                del_end -= n
                variant = variant[:-n]

        # due to gaps, the variant is not actually within the feauture
        if del_start >= del_end: return        

        refseq = sequence[del_start:del_end].upper()

        assert refseq == variant, \
            'reference base mismatch at deletion: expected %s %s %s, got %s[%i:%i] at feature=%i-%i, variant=%i-%i, relative=%i-%i, del=%i-%i, action=%s' % \
            (sequence[del_start-10:del_start],
             refseq, 
             sequence[del_end:del_end+10],
             variant, startoffset, len(variant)-endoffset,
             feature_start, feature_end,
             var_start, var_end,
             rel_start, rel_end,
             del_start, del_end,
             action)

        l = del_end - del_start

        # assert len("".join(allele[del_start:del_end])) == l, \
        #     "deletion conflicts with other indels: " \
        #     "got %s[%i:%i] (ref=%s, allele=%s) at feature=%i-%i, variant=%i-%i, relative=%i-%i, del=%i-%i, action=%s" % \
        #     (variant, startoffset, len(variant)-endoffset,
        #      refseq, str(allele[del_start:del_end]),
        #      feature_start, feature_end,
        #      var_start, var_end,
        #      rel_start, rel_end,
        #      del_start, del_end,
        #      action)

        allele[del_start:del_end] = [""] * l    

    allele1 = list( sequence.lower() )
    allele2 = list( sequence.lower() )
    
    if reference_start == None:
        feature_start = 0
    else:
        feature_start = reference_start

    feature_end = feature_start + len(sequence)

    # main loop: insert variants into allele sequences
    for var_start, var_end, reference, action, has_wildtype, variantseqs in variants:

        # skip variants that are out-of-range
        if var_end <= feature_start or var_start >= feature_end: continue

        is_homozygous = len(variantseqs) == 1 and not has_wildtype

        rel_start, rel_end = var_start - feature_start, var_end - feature_start
        startoffset = max(0, feature_start-var_start)
        endoffset = max(0,var_end-feature_end)
        pruned_start, pruned_end = max(0,rel_start), min(len(sequence),rel_end)

        if action == "=":

            if E.global_options.loglevel >= 10:
                E.debug( "adding SNP at postition %i: reference=%s variants=%s" % (var_start, reference, variantseqs))

            if allele1[rel_start] == "" or allele2[rel_start] == "":
                # these can be cases, where a base is deleted in one allele,
                # but recorded as a homozygous substitution in another allele.
                E.warn( "substitution conflicts with a deletion - ignored: %s" % str( (var_start, var_end, reference, action, has_wildtype, variantseqs)) )
                continue

            assert rel_start >= 0
            assert sequence[rel_start].upper() == reference, \
                'reference base mismatch: expected %s %s %s, got %s at feature=%i-%i, variant=%i-%i, relative=%i-%i, pruned=%i-%i, action=%s' % \
                (sequence[rel_start-10:rel_start],
                 sequence[rel_start].upper(),
                 sequence[rel_start+1:rel_start+10],
                 reference,
                 feature_start, feature_end,
                 var_start, var_end,
                 rel_start, rel_end,
                 pruned_start, pruned_end,
                 action)

            if phased:
                allele1[rel_start] = variantseqs[0] + allele1[rel_start][1:]
                allele2[rel_start] = variantseqs[1] + allele2[rel_start][1:]
            elif is_homozygous:
                allele1[rel_start] = variantseqs[0] + allele1[rel_start][1:]
                allele2[rel_start] = variantseqs[0] + allele2[rel_start][1:]
            else:
                if has_wildtype:
                    if reference == variantseqs[0]:
                        allele2[rel_start] = variantseqs[1] + allele2[rel_start][1:]
                    else:
                        allele2[rel_start] = variantseqs[0] + allele2[rel_start][1:]
                else:
                    allele1[rel_start] = variantseqs[0] + allele1[rel_start][1:]
                    allele2[rel_start] = variantseqs[1] + allele2[rel_start][1:]

        elif action == "-":
            if phased:
                _delete( allele1, pruned_start, pruned_end, variantseqs[0], 
                         sequence, startoffset, endoffset, feature_start, feature_end )
                _delete( allele2, pruned_start, pruned_end, variantseqs[1], 
                         sequence, startoffset, endoffset, feature_start, feature_end )
            elif is_homozygous:
                _delete( allele1, pruned_start, pruned_end, variantseqs[0], 
                         sequence, startoffset, endoffset, feature_start, feature_end )
                _delete( allele2, pruned_start, pruned_end, variantseqs[0], 
                         sequence, startoffset, endoffset, feature_start, feature_end )
            else:
                if has_wildtype:
                    _delete( allele2, pruned_start, pruned_end, variantseqs[0], 
                             sequence, startoffset, endoffset, feature_start, feature_end )
                else:
                    _delete( allele1, pruned_start, pruned_end, variantseqs[0], 
                             sequence, startoffset, endoffset, feature_start, feature_end )
                    _delete( allele2, pruned_start, pruned_end, variantseqs[1], 
                             sequence, startoffset, endoffset, feature_start, feature_end )

        elif action == "+":
            # ignore insertions at position -1
            if rel_start < 0: continue

            if phased:
                allele1[rel_start] += variantseqs[0].upper()
                allele2[rel_start] += variantseqs[1].upper()
            elif is_homozygous:
                allele1[rel_start] += variantseqs[0].upper()
                allele2[rel_start] += variantseqs[0].upper()
            else:
                if has_wildtype:
                    allele2[rel_start] += variantseqs[0].upper()
                else:
                    allele1[rel_start] += variantseqs[0].upper()
                    allele2[rel_start] += variantseqs[1].upper()

        elif action == ">":
            # indel
            if rel_start >= 0: allele1[rel_start] += variantseqs[0].upper()
            _delete( allele2, pruned_start, pruned_end, variantseqs[1], 
                     sequence, startoffset, endoffset, feature_start, feature_end )


        elif action == "<":
            #delin
            if rel_start >= 0: allele2[rel_start] += variantseqs[1].upper()
            _delete( allele1, pruned_start, pruned_end, variantseqs[0], 
                     sequence, startoffset, endoffset, feature_start, feature_end )

    assert len(sequence) == len(allele1)
    assert len(sequence) == len(allele2)

    return (allele1, allele2)

def buildOffsets( variants, phased = True, contig = None ):
    '''collect coordinate offsets.

    This methods takes a set of variants and computes
    coordinates offsets based on indels.

    Conflicting variants will be removed. 

    Returns a list of variants, a list of removed variants and a list of offsets.
    '''

    variants.sort()
    offset0, offset1 = 0, 0
    offsets0, offsets1 = [(0,0)], [(0,0)]

    def addDeletion( offsets, start, end, offset, variantseq ):
        
        for x, c in enumerate(variantseq):
            if c != "-": 
                offset -= 1
                offsets.append( (start+x+1,offset) )

        return offset

    last_end = 0

    removed_variants, kept_variants = [], []
    for variant in variants:
    
        start, end, reference, action, has_wildtype, variantseqs = variant

        if action == "=": 
            kept_variants.append( variant )
            # ignore substitutions
            continue

        if last_end >= start:
            removed_variants.append( variant )
            continue

        kept_variants.append( variant )
        is_homozygous = len(variantseqs) == 1 and not has_wildtype

        last_end = end
        if action == "-":
            if phased:
                offset0 = addDeletion( offsets0, start, end, offset0, variantseqs[0] )
                offset1 = addDeletion( offsets1, start, end, offset1, variantseqs[1] )
            elif is_homozygous:
                offset0 = addDeletion( offsets0, start, end, offset0, variantseqs[0] )
                offset1 = addDeletion( offsets1, start, end, offset1, variantseqs[0] )
            else:
                if has_wildtype:
                    offset1 = addDeletion( offsets1, start, end, offset1, variantseqs[0] )
                else:
                    offset0 = addDeletion( offsets0, start, end, offset0, variantseqs[0] )
                    offset1 = addDeletion( offsets1, start, end, offset1, variantseqs[1] )

        elif action == "+":

            if phased:
                offset0 += len(variantseqs[0]) - variantseqs[0].count("-")
                offset1 += len(variantseqs[1]) - variantseqs[1].count("-")
            elif is_homozygous:
                offset0 += len(variantseqs[0])
                offset1 += len(variantseqs[0])
            else:
                if has_wildtype:
                    offset1 += len(variantseqs[0])
                else:
                    offset0 += len(variantseqs[0]) - variantseqs[0].count("-")
                    offset1 += len(variantseqs[1]) - variantseqs[1].count("-")
            offsets0.append((end-1,offset0))
            offsets1.append((end-1,offset1))

        elif action == ">":
            offset0 += len(variantseqs[0]) - variantseqs[0].count("-")
            offsets0.append((end-1,offset0))
            offset1 = addDeletion( offsets1, start, end, offset1, variantseqs[1] )

        elif action == "<":
            offset0 = addDeletion( offsets0, start, end, offset0, variantseqs[0] )
            offset1 += len(variantseqs[1]) - variantseqs[1].count("-")
            offsets1.append((end-1,offset1))
        
        if E.global_options.loglevel >= 8:
            print "# variant=",start, end, reference, action, has_wildtype, variantseqs
            print "# offsets:", offsets0[-1], offsets1[-1]
        # add offsets - applies to the end of the variant
            


    return ( kept_variants, removed_variants, (offsets0, offsets1))



################################################################################
#
#   Gene prediction pipeline 
#
#   $Id: trees2sets.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
trees2sets.py - compute ortholog sets from trees
================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

extract sets from a list of trees. The trees need to be rooted already.

Given a reference tree and a collection of trees, this script outputs
(depending on the option --method)

   * strict: 1:1 ortholog clusters for a selected set of species
     combinations. All species need to be present for a species combination
   
   * degenerate: 1:m ortholog cluster for a selected set of species
     combinations. All species need to be present for a species combination.

   * lineage: lineage specific duplications.

   * any: 1:1 and 1:m ortholog clusters

The default set of species combinations consists of all monophyletic clades.
Different sets can be specified with the option --enumeration:

   * exhaustive: all possible species combinations
   
   * monophyletic: all monophyletic species combinations (provide a species tree)
   
   * full: one set of all species

   * explicit: species combination given on command line

   * lineage: lineage specific duplications

   * pairwise: all pairwise species combinations

The script can map strains to species.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

""" 

import os
import sys
import string
import re
import getopt
import optparse

import CGAT.Orthologs as Orthologs

USAGE="""python trees2sets.py < stdin 


"""

import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools
import CGAT.SetTools as SetTools
import CGAT.IOTools as IOTools

def buildPattern( nspecies, s, members = None):
    """build a present/absent pattern for species
    in set s.
    """
    pattern = []
    for x in range(nspecies):
        if x in s:
            if members:
                pattern.append( str(len(members[x])) )
            else:
                pattern.append( "1" )                
        else:
            pattern.append( "0" )
    return pattern

def parseIdentifier( id, options ):

    data = id.split(options.separator)
    if len(data) == 4:
        return data
    elif len(data) == 2:
        s,g = data
        t = g
        q = "UK"
        return s,t,g,q

##--------------------------------------------------------------------------------
def getOrthologNodes( tree, positive_set, options, selector = "strict", outgroups = None ):
    """get all ortholog nodes in tree for species in positive_set.

    Depending on the selector function, different sets are returned:
    
    If selector is "strict", only strict orthologs are returned. These
    contain exactly one gene per species for all species in the positive_set.
    
    If selector is "degenerate", only degenerate orthologs are returned.
    These contain at least gene per species for species in the positive_set.
    
    Collect genes in tree for each species.

    Returns the node_id for which a set fulfills the criteria
    and the set for which it fulfills it.

    Avoid double counting: if you are interested in species A and B,
    any branches involving others species should be ignored. Make sure to only
    count once and not every time a discarded branch is removed.

    Thus, as soon as A and B merge, any node further up the tree have to be
    ignored.

    total_genes_function:   if true, node is recorded
    total_species_function: if true, iteration stops

    """

    nspecies = len(options.org2column)
    
    if selector == "strict":
        ## strict orthologs: at most one gene per species
        exit_function = lambda num_genes_for_species: num_genes_for_species > 1
        keep_function = lambda num_genes_for_species: num_genes_for_species == 1
        total_genes_function = lambda num_genes_at_node,num_species_in_pattern: num_genes_at_node == num_species_in_pattern
        total_species_function = lambda num_species_at_node,num_species_in_pattern: num_species_at_node == num_species_in_pattern
        check_outgroup_function = lambda x: False        
        negative_set = set()
    elif selector == "degenerate":
        ## degenerate orthologs: any number of genes per species,
        exit_function = lambda num_genes_for_species: False
        keep_function = lambda num_genes_for_species: num_genes_for_species > 0
        total_genes_function = lambda num_genes_at_node,num_species_in_pattern: num_genes_at_node > num_species_in_pattern
        total_species_function = lambda num_species_at_node,num_species_in_pattern: num_species_at_node == num_species_in_pattern        
        check_outgroup_function = lambda x: False
        negative_set = set()
    elif selector == "lineage":
        ## lineage specific duplications: at least 1 gene
        exit_function = lambda num_genes_for_species: False
        keep_function = lambda num_genes_for_species: num_genes_for_species > 1
        total_genes_function = lambda num_genes_at_node,num_species_in_pattern: num_genes_at_node >= num_species_in_pattern
        total_species_function = lambda num_species_at_node,num_species_in_pattern: False
        check_outgroup_function = lambda x: False
        negative_set = set( range(nspecies) ).difference( positive_set )
    elif selector == "any":
        ## any number of orthologs, including
        ## orphans
        exit_function = lambda num_genes_for_species: False
        keep_function = lambda num_genes_for_species: True
        total_genes_function = lambda num_genes_at_node,num_species_in_pattern: True
        total_species_function = lambda num_species_at_node,num_species_in_pattern: num_species_at_node == num_species_in_pattern
        check_outgroup_function = lambda x: False
        negative_set = set()
    elif selector == "outgroup":
        ## group selector
        exit_function = lambda num_genes_for_species: False
        keep_function = lambda num_genes_for_species: num_genes_for_species > 0
        total_genes_function = lambda num_genes_at_node,num_species_in_pattern: False        
        total_species_function = lambda num_species_at_node,num_species_in_pattern: False
        ## check for outgrup: needs to have outgroup and at least one other species
        ## ie.: sum of all genes in outgroups larger than sum of all genes in all species
        if not outgroups:
            raise "usage error: please supply outgroups if 'outgroup'-selector is chosen."
        check_outgroup_function = lambda genes: 0 < sum( [ len(genes[x]) for x in outgroups ] ) < sum( map( lambda x: len(x), genes) )
        negative_set = set()
    else:
        raise "unknown selector %s" % selector

    ## work here: set genes[node_id] to None,
    ## 1. if the gene count for a species of interest is > 1
    ## 2. if the gene count for all species of interest is 1 in
    ## the child node.

    if options.loglevel >= 5:
        options.stdlog.write( "# gene tree\n" )
        tree.display()

    n = TreeTools.GetSize( tree ) + 1
    genes = []
    for x in range(n):
        genes.append( [ set() for x in range(nspecies)] )

    ortholog_nodes = []

    def count_genes( node_id ):
        """record number of genes per species for each node
        """
        node = tree.node(node_id)
        
        if options.loglevel >= 6:
            options.stdlog.write("# node_id=%i\n" % node_id)
            if options.loglevel >= 10:
                options.stdlog.write("# sets=%s\n" % (str(genes)))

        # species in pattern
        num_species_in_pattern = len(positive_set)
        
        if node.succ:
            ## process non-leaf node
            for s in node.succ:

                ## propagate: terminated nodes force upper nodes to terminate (assigned to None).
                if not genes[s]:
                    genes[node_id] = None
                    return
                
                ## total number of genes at node
                num_genes_at_node = 0
                ## total number of species at node
                num_species_at_node = 0

                ## compute new gene set for each species at node
                for x in positive_set:
                    genes[node_id][x] = genes[node_id][x].union(genes[s][x])
                    
                    num_genes_for_species = len(genes[node_id][x])
                    if exit_function(num_genes_for_species):
                        genes[node_id] = None
                        return
                    num_genes_at_node += num_genes_for_species
                    if num_genes_for_species: num_species_at_node += 1
                    
            if options.loglevel >= 6:
                print "node=", node_id, "species_at_node", num_species_at_node, "genes_at_node=", num_genes_at_node, \
                    "num_genes_for_species=", num_genes_for_species, "ngenes=", sum( map( lambda x: len(x), genes[node_id]) )
                options.stdlog.write("# genes at node %i\t%s\n" % (node_id, genes[node_id]))
                if outgroups:
                    print sum( [ len(genes[node_id][x]) for x in outgroups ] )
                    print check_outgroup_function(genes[node_id])

            # check stop criterion
            if total_species_function( num_species_at_node, num_species_in_pattern ):
                ## check if positive requirements are fulfilled
                for x in positive_set:
                    if not keep_function(len(genes[node_id][x])):
                        if options.loglevel >= 6:
                            options.stdlog.write("# keep function false for species %i\n" % x)
                        break
                else:
                    if total_genes_function( num_genes_at_node, num_species_in_pattern):
                        if options.loglevel >= 6:
                            options.stdlog.write("# recording node %i\n" % x)
                        ortholog_nodes.append( (node_id, genes[node_id]) )
                genes[node_id] = None
                return
            elif check_outgroup_function( genes[node_id] ):
                ortholog_nodes.append( (node_id, genes[node_id]) )
                genes[node_id] = None
                return
            elif negative_set:
                if total_genes_function( num_genes_at_node, num_species_in_pattern):
                    if options.loglevel >= 6:
                        options.stdlog.write("# recording node %i\n" % node_id)
                    ortholog_nodes.append( (node_id, genes[node_id]) )

        else:
            ## process leaf
            s, t, g, q = parseIdentifier( node.data.taxon, options )
            c = options.org2column[s]
            if c in positive_set:
                genes[node_id][c].add( g )
            elif c in negative_set:
                genes[node_id] = None

    tree.dfs( tree.root, post_function = count_genes )
    
    return ortholog_nodes

##------------------------------------------------------------------------------------------                
##------------------------------------------------------------------------------------------                
##------------------------------------------------------------------------------------------
def rerootTree( gene_tree, extract_species, options ):
    """reroot tree."""

    otus = TreeTools.GetTaxa( gene_tree )
    
    ## find monophyletic trees of outgroup_species
    try:
        outgroup_taxa = filter( lambda x: extract_species(x) in options.outgroups, otus)
    except AttributeError:
        raise "error while rerooting tree in tree %s with %s" % (gene_tree.name, str(otus))

    if gene_tree.is_monophyletic( outgroup_taxa ):
        r = outgroup_taxa
    else:
        r = [outgroup_taxa[0],]

    if r:
        if options.loglevel >= 1:
            options.stdlog.write("# tree %s: rerooting with %i outgroups:  %s.\n" % (gene_tree.name, len(r), ",".join(r)))
            options.stdlog.flush()            
    else:
        if options.loglevel >= 1:
            options.stdlog.write("# tree %s: no outgroup found, tree will not be rerooted.\n" % gene_tree.name)
            options.stdlog.flush()
            
    gene_tree.root_with_outgroup( r )

    if options.loglevel >= 5:
        gene_tree.display()

##--------------------------------------------------------------------------------
def writeOrthologSets( outfile, nexus,
                       extract_species,
                       extract_gene,
                       options,
                       reference_tree = None,
                       method = "strict",
                       outgroups = None ):
    """output ortholog sets.

    A "strict" ortholog set contains exactly one gene for each species,
    while a "degenerate" ortholog set contains at least one gene for each
    species. 
    """

    ######################################################################
    # build species set to compare
    sets = []
    species = options.column2org
    nspecies = len(species)
    
    if options.enumeration == "monophyletic":
        if reference_tree:
            for members, h1, h2 in TreeTools.GetSubsets( reference_tree ):
                if len(members) > 1:
                    sets.append( members )
        else:
            raise "please specify a species tree for monophyletic enumeration"
        
    elif options.enumeration == "exhaustive":
        for x in range(2, len(species) ):
            sets += list(SetTools.xuniqueCombinations( species, x))
        sets.append(species)
                
    elif options.enumeration == "pairwise":
        
        for x in range(len(species)-1):
            for y in range(x+1, len(species)):
                sets.append( (species[x], species[y]) )
                
    elif options.enumeration == "full":                
        sets.append( species )

    elif options.enumeration == "lineage":
        for s in species: 
            sets.append( (s,) )

    elif options.enumeration == "explicit":
        for x in range(2, len(options.species_set)):
            sets += list(SetTools.xuniqueCombinations( options.species_set, x ))
        sets.append(options.species_set)

    ######################################################################
    # build sets with positional information
    xsets = []
    map_frozenset2set = {}
    for x in range(len(sets)):
        ss = frozenset(map( lambda x: options.org2column[x], sets[x]) )
        xsets.append( ss )
        map_frozenset2set[ss] = x

    ######################################################################
    ## collect outgroups
    if outgroups:
        noutgroups = set()
        for x in outgroups:
            noutgroups.add( options.org2column[x] )
    else:
        noutgroups = None
        
    ######################################################################
    # loop over each tree and set
    # I did not see a way to loop a tree once for all sets without doing
    # complicated counting. The problem is that counting has to be stopped
    # at different tree heights for different sets.
    ninput, noutput, nempty, nskipped = 0, 0, 0, 0

    counts = [0] * len(sets)

    options.stdout.write( "nspecies\tname\tid\tcluster\tpattern\t%s\tnode_id\tmembers\n" % "\t".join(species))

    cluster_id = 0
    nerrors = 0
    
    for tree in nexus.trees:
        
        ninput += 1
        ntotal_tree = 0

        if options.loglevel >= 3:
            options.stdlog.write("# processing tree %s\n" % tree.name )

        if options.reroot:
            rerootTree( tree, extract_species, options )
        
        for c in range(len(xsets)):
            ## numbered species set: 0,1,...
            sn = xsets[c]
            ## literal species set: species1, species2, ...
            sl = sets[c]
            
            ortholog_nodes = getOrthologNodes( tree, sn, options, selector = method,
                                               outgroups = noutgroups )
            ntotal_tree += len(ortholog_nodes)

            n = 0

            pattern =  buildPattern( nspecies, sn )

            ## check for inconsistent partitions (the same gene in different
            ## ortholog clusters) within the current tree
            found_genes = set()
            ortho_sets = set()
            
            ## reverse ortholog_node - work in top-down manner.
            ortholog_nodes.reverse()

            for node_id, members in ortholog_nodes:
                n += 1
                cluster_id += 1
                
                otus = filter( lambda x: extract_species(x) in sl, tree.get_taxa( node_id ) )
                genes = set( map( extract_gene, otus ) )

                if found_genes.intersection( genes ):

                    ## only take largest cluster for lineage specific duplications
                    if method == "lineage":
                        continue

                    if frozenset(genes) in ortho_sets:
                        nskipped += 1
                        if options.loglevel >= 1:
                            options.stdlog.write("# %s: cluster %i: redundant node: %i - skipped because already present: %s\n" %
                                                 (tree.name, n, node_id, str(found_genes.intersection( genes ))))
                    else:
                        nerrors += 1
                        if options.loglevel >= 1:
                            options.stdlog.write("# %s: cluster %i: inconsistent node: %i - the same gene in different clusters: %s\n" %
                                                 (tree.name, n, node_id, str(found_genes.intersection( genes ))))
                    
                found_genes = found_genes.union( genes )
                ortho_sets.add( frozenset(genes) )
                
                xpattern = buildPattern( nspecies, sn, members )
                
                options.stdout.write( "%i\t%s\t%i\t%i\t%s\t%s\t%i\t%s\n" % (len(sl),
                                                                            tree.name,
                                                                            n,
                                                                            cluster_id, 
                                                                            "".join(pattern),
                                                                            "\t".join(xpattern),
                                                                            node_id,
                                                                            ";".join(otus) ))
                
            counts[c] += n
            
        if ntotal_tree == 0:
            nempty += 1
        else:
            noutput += 1

    if options.loglevel >= 1:
        options.stdout.write("# ninput=%i, nempty=%i, noutput=%i, nskipped=%i, nerrors=%i\n" % (ninput, nempty, noutput, nskipped, nerrors) )

    ## write summary information
        
    if options.filename_summary:
        outfile = open(options.filename_summary, "w")
    else:
        outfile = options.stdout
        outfile.write("//\n")
        
    outfile.write( "cluster\tpattern\tcounts\t%s\n" % ("\t".join(species)) )
    
    for c in range(len(xsets)):
        pattern = buildPattern( nspecies, xsets[c])
        outfile.write("%i\t%s\t%i\t%s\n" % ( c,
                                             "".join(pattern),
                                             counts[c],
                                             "\t".join(pattern) ))
                                             
    if outfile != options.stdout:
        outfile.close()


if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: trees2sets.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])
    
    parser.add_option("-t", "--reference-tree", dest="reference_tree", type="string",
                      help="reference tree to read."  )

    parser.add_option("-e", "--enumeration", dest="enumeration", type="choice",
                      choices=("monophyletic", "full", "pairwise", "exhaustive", "explicit", "lineage" ),
                      help="enumeration of ortholog groups."  )

    parser.add_option( "-o", "--organisms", dest="column2org", type="string" ,
                       help="sorted list of organisms.")

    parser.add_option("-p", "--filename-patterns", dest="filename_patterns", type="string",
                      help="filename with patterns to output."  )

    parser.add_option("-u", "--filename-summary", dest="filename_summary", type="string",
                      help="filename with summary to output."  )

    parser.add_option("-m", "--method", dest="methods", type="choice", action="append",
                      choices=("strict", "degenerate", "any", "outgroup", "lineage" ),
                      help="sets to extract."  )

    parser.add_option("-s", "--species-set", dest="species_set", type="string",
                      help="comma separated list of species." )

    parser.add_option("-g", "--outgroups", dest="outgroups", type="string",
                      help="comma separated list of outgroup species." )

    parser.add_option( "--species-regex", dest="species_regex", type="string" ,
                       help="regular expression to extract species from identifier.")

    parser.add_option( "--gene-regex", dest="gene_regex", type="string" ,
                       help="regular expression to extract gene from identifier.")

    parser.add_option( "--reroot", dest="reroot", type="choice" ,
                       choices=("outgroup", "midpoint"),
                       help="reroot trees before computing sets.")

    parser.set_defaults(
        reference_tree = None,
        enumeration = "full",
        column2org=None,
        separator = "|",
        species_regex ="^([^|]+)\|",
        gene_regex = "^[^|]+\|[^|]+\|([^|]+)\|",
        filename_summary = None,
        methods = [],
        species_set = None,
        outgroups = None,
        reroot = None,
        )

    (options, args) = E.Start( parser )

    if len(options.methods) == 0:
        options.methods.append("strict")

    if options.species_set:
        options.species_set = options.species_set.split(",")
        options.enumeration = "explicit"

    #######################################################################
    ## warning: outgroup method is useless, as it requires
    ## only a single outgroup per tree and the tree rooted
    ## with the outgroup.
    if "outgroup" in options.methods and not options.outgroups:
        raise "please supply --outgroups if method 'outgroup' is chosen."

    if options.outgroups:
        options.outgroups = options.outgroups.split(",")
        
    ########################################################################
    ########################################################################
    ########################################################################
    if options.reference_tree:
        if options.reference_tree[0] == "(":
            nexus = TreeTools.Newick2Nexus( options.reference_tree )
        else:
            nexus = TreeTools.Newick2Nexus( open(options.reference_tree, "r") )
        reference_tree = nexus.trees[0]
    
        if options.loglevel >= 3:
            options.stdlog.write("# reference tree:\n%s\n" % reference_tree.display())
    else:
        reference_tree = None
        raise ValueError( "please supply a reference tree" )

    ########################################################################
    ########################################################################
    ########################################################################
    ## read all trees
    ########################################################################    
    nexus = TreeTools.Newick2Nexus( sys.stdin )

    ########################################################################
    ########################################################################
    ########################################################################
    ## sort out reference tree
    ########################################################################
    rs = re.compile( options.species_regex )
    rg = re.compile( options.gene_regex )
    extract_species = lambda x: parseIdentifier(x, options)[0]
    extract_gene = lambda x:parseIdentifier(x, options)[2]
    
    ## prune reference tree to species present
    species_set = set()
    for tree in nexus.trees:
        try:
            species_set = species_set.union( set(map( extract_species, tree.get_taxa()) ) )
        except AttributeError:
            raise "parsing error while extracting species from %s" % str(tree.get_taxa())

    TreeTools.PruneTree( reference_tree, species_set )
    
    if options.loglevel >= 1:
        options.stdlog.write("# reference tree after pruning has %i taxa.\n" % len(reference_tree.get_taxa()))

    if options.column2org:
        options.column2org = options.column2org.split(",")
    elif reference_tree:
        options.column2org = []
        for nx in reference_tree.get_terminals():
            options.column2org.append( reference_tree.node(nx).get_data().taxon )
    
    options.org2column = {}
    for x in range(len(options.column2org)):
        options.org2column[options.column2org[x]] = x

    for method in options.methods:

            #######################################################################################
            #######################################################################################
            #######################################################################################
            ## print out a list of ortholog clusters
            #######################################################################################
            writeOrthologSets( options.stdout,
                               nexus,
                               extract_species,
                               extract_gene,
                               options = options,
                               reference_tree = reference_tree,
                               method = method,
                               outgroups = options.outgroups )
    
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gff2view.py - view gff intervals in UCSC genome browser
=======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads intervals from a :term:`gff` formatted file and
opens the corresponding location in the UCSC genome browser. If there
are multiple intervals, multiple windows/tabs will be opened, so be
careful.

Usage
-----

Example::

   python gff2view.py --help

Type::

   python gff2view.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time
import tempfile
import subprocess

import webbrowser

USAGE="""python %s [OPTIONS] < stdin

open a browser window at several locations. This script interfaces
with UCSC and genome browser.

TODO: add support for ensembl
""" % sys.argv[0]

import CGAT.GTF as GTF
import CGAT.Experiment as E
import CGAT.IndexedFasta as IndexedFasta

##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: gff2view.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-t", "--target", dest="target", type="choice",
                      choices = ("ucsc", "gbrowser"),
                      help="target location to open [%default]."  )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome."  )

    parser.add_option( "--is-gtf", dest="is_gtf", action="store_true",
                      help="input is gtf."  )

    parser.add_option("-f", "--add-flank", dest="flank", type="int",
                      help="add # nucleotides for each region."  )

    parser.add_option("-z", "--zoom", dest="zoom", type="float",
                      help="zoom out (# > 1) or in (# < 1)."  )

    parser.add_option("-c", "--chunk-size", dest="chunk_size", type="int",
                      help="number of tabs to display in one go."  )

    parser.add_option( "--ucsc-assembly", dest="ucsc_assembly", type="string",
                      help="ucsc assembly."  )

    parser.add_option( "--ucsc-user-tracks", dest="ucsc_user_tracks", type="string",
                      help="ucsc user tracks."  )

    parser.add_option( "--gbrowser-assembly", dest="gbrowser_assembly", type="string",
                      help="gbrowser assembly."  )

    parser.add_option( "--randomize", dest="randomize", action="store_true",
                       help="randomize input [%default]" )
    
    parser.set_defaults(
        ucsc_assembly = "ponAbe2",
        ucsc_url = "http://genome.ucsc.edu/cgi-bin/hgTracks",
        gbrowser_assembly = "Songbird",
        gbrowser_url = "http://genserv.anat.ox.ac.uk/cgi-bin/devel/gbrowse",
        genome_file = None,
        ucsc_custom_annotation = "http://wwwfgu.anat.ox.ac.uk/~andreas/ucsc_tracks/%s",
        ucsc_user_tracks = None,
        flank = None,
        zoom = None,
        chunk_size = 50,
        is_gtf = False,
        target = "ucsc",
        randomize = False, 
        joined = False,
        )

    (options, args) = E.Start( parser )

    if len(args) != 1:
        print USAGE
        raise "please specify the gff file to open."
    
    if options.is_gtf:
        entry_iterator = GTF.iterator
        chunk_iterator = GTF.flat_gene_iterator
    else:
        entry_iterator = GTF.iterator
        if options.joined:
            chunk_iterator = GTF.joined_iterator
        else:
            chunk_iterator = GTF.chunk_iterator

    if len(args) == "0" or args[0] == "-":
        iterator = chunk_iterator( entry_iterator( sys.stdin) )
    else:
        iterator = chunk_iterator( entry_iterator( open(args[0], "r") ) )

    nopened = 0

    # b = webbrowser.get( "konqueror" )
    b = webbrowser.get( "firefox" )

    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )
    else:
        fasta = None

    if options.ucsc_user_tracks:
        annotations = "hgt.customText=%s" % (options.ucsc_custom_annotation % options.ucsc_user_tracks)
    else:
        annotations = None

    for chunk in iterator:
        start = min( [ x.start for x in chunk ] )
        end = max( [ x.end for x in chunk ] )

        if options.flank: 
            start -= options.flank
            end += options.flank
            
        if options.zoom:
            s = end - start
            d = options.zoom * s - s
            start -= d
            end += d
            
        start = max( 0, start )

        contig = chunk[0].contig

        if fasta:
            contig = fasta.getToken( contig )
            end = min( end, fasta.getLength( contig ) )
            
        if len(contig) < 3: contig = "chr%s" % contig

        if options.target == "ucsc":
            url_options = [ "db=%s" % options.ucsc_assembly,
                            "position=%s:%i-%i" % (contig, start, end) ]

            if annotations: url_options.append( annotations )

            url = "%s?%s" % (options.ucsc_url, 
                             "&".join( url_options) )
        elif options.target == "gbrowser":
            url = "%s/%s?name=%s:%i..%i" % (options.gbrowser_url, 
                                            options.gbrowser_assembly,
                                            contig, 
                                            start, 
                                            end )

        print "# opening browser window for:"
        print "#", url

        if nopened % options.chunk_size == 0: 
            if nopened != 0:
                x = raw_input('showing %i - hit return to continue:' % options.chunk_size )
            b.open_new(url)
            first = False
        else:
            b.open_new_tab( url )

        nopened += 1
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
data2multiple_anova.py - compute multiple ANOVA between samples
===============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import sys
import re
import string
import os
import optparse
import math

import CGAT.Experiment as E
import scipy
import scipy.stats
import CGAT.IOTools as IOTools

import rpy2
from rpy2.robjects import r as R

##---------------------------------------------------------------------------------------------------------        
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: data2multiple_anova.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to take for calculating histograms." )
    parser.add_option("-t", "--filename-tree", dest="filename_tree", type="string",
                      help="filename with tree(s)." )
    parser.add_option("--skip-header", dest="add_header", action="store_false",
                      help="do not add header to flat format."  )
    parser.add_option("--write-header", dest="write_header", action="store_true",
                      help="write header and exit."  )
    parser.add_option("--debug", dest="debug", action="store_true",
                      help="debug mode")
    parser.add_option("--display-tree", dest="display_tree", action="store_true",
                      help="display the tree")
    
    parser.add_option("-m", "--method", dest="methods", type="choice", action="append",
                      choices=("contrasts", "spearman", "pearson", "compute" ),
                      help="methods to perform on contrasts." )
    
    parser.set_defaults(
        columns = "all",
        filename_tree = None,
        add_header = True,
        write_header = False,
        debug = False,
        methods = [],
        value_format = "%6.4f",
        pvalue_format = "%e",
        display_tree = False,
        )

    (options, args) = E.Start( parser, quiet = True )

    if options.columns not in ( "all", "all-but-first"):
        options.columns = map(lambda x: int(x) -1 , options.columns.split(","))

    data = []

    options.filenames = args

    for filename in options.filenames:
        
        infile = open(filename,"r")
        table, headers = IOTools.readTable( infile, take = options.columns, headers=False)
        infile.close()

        data.append( table )
        

    fields = [ "Df", "Sum Sq", "F value", "Pr(>F)", "Mean Sq"]
    
    options.stdout.write("set1\tset2" )
    for field in fields:
        options.stdout.write("\t%s" % field )
    options.stdout.write("\n" )

    # CODE needs to be refactored for rpy2 usage

    for x in range( len(data )):

        for y in range(x+1,len(data)):

            rpy.set_default_mode(rpy.NO_CONVERSION)
            
            factors = ["x"] * len(data[x][:,0]) + ["y"] * len(data[y][:,0])
            values = list(data[x][:,0]) + list(data[y][:,0])

            linear_model = R.lm(R("y ~ x"), data = R.data_frame(x=factors, y=values ))
            rpy.set_default_mode(rpy.BASIC_CONVERSION)
            result = R.anova( linear_model )
            
            options.stdout.write( "%s\t%s" % (options.filenames[x], options.filenames[y]) )
            for field in fields:
                options.stdout.write("\t%s" % str( result[field] ) )
            options.stdout.write("\n" )







################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
go2plot.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python go2plot.py --help

Type::

   python go2plot.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

""" program $Id: go2plot.py 2782 2009-09-10 11:40:29Z andreas $

"""
import CGAT.Experiment as E

doEps = True                 # generate eps instead of ps (but make sure it's just one page!)
largepvalues = True          # if true smallest p value is 10^-4 otherwise 10^-5.
annsize = 90                 # width of annotation text column
colsize = 9                  # width of column
colsep = 2                   # separation between (small) columns
rowheight = 7                # height of line
rowwidth = rowheight         # used for heading text only (?)
textangle = 45               # angle of heading text
labelsize = 70               # upper margin used for printing column labels
maincolsep = 15              # whitespace between (big) columns
greylines = 5                # number of lines per grey divider line
maxp = 1.0                   # only include annotations with p value < maxp in some input file
vertical = 0                 # if nonzero, prints a vertical grey line at (small) column 'vertical'

maxsize = rowheight-1
maxheight = 815 - labelsize - 50
maxwidth = 585 - labelsize/3 - 10

if largepvalues:
    # minimum 1/10,000
    pval0 = 0.0001
    pval1 = 0.0003
    pval2 = 0.001
    pval3 = 0.003
    pval4 = 0.01
    pval5 = 0.03
else:
    # minimum 1/100,000
    pval0 = 0.00001
    pval1 = 0.0001
    pval2 = 0.001
    pval3 = 0.01
    pval4 = 0.1
    pval5 = 0.3


def posx(x):
    return x*(colsize+colsep)+annsize

def posy(y):
    return (collen-y)*rowheight

def makeheader( input, annlist, numcols, numpages ):
    if doEps:
        print "%!PS-Adobe-2.0 EPSF-2.0"
    else:
        print "%!PS-Adobe-2.0"
        print "%%Pages: ",numpages
    print "%%BoundingBox: ", \
          10, \
          posy(collen+4)+60, \
          int(10+(numcols-1)*maincolsep + labelsize/3 + (numcols)*posx(len(input))),\
          int(posy(0)+labelsize)+60
    print """
/makebox {
    % x y size color
    gsave
    setrgbcolor
    /size exch def
    /y exch def
    /x exch def
     x y translate
     size size scale
     newpath
     -0.5 -0.5 moveto
     1 0 rlineto
     0 1 rlineto
     -1 0 rlineto
     0 -1 rlineto
     closepath
     fill
     grestore
} def
/min {/minarg1 exch def /minarg2 exch def
     minarg1 minarg2 lt {minarg1} {minarg2} ifelse}def
/max {/maxarg1 exch def /maxarg2 exch def
     maxarg1 maxarg2 gt {maxarg1} {maxarg2} ifelse}def
"""
    

def sameannotation(ann1,ann2):
    return ann1.replace('+','').replace('-','') == ann2.replace('+','').replace('-','')


def makecolheader(input, annlist, title = ""):
    print "/Times-Roman findfont ",colsize," scalefont setfont"
    for i in range(len(input)):
        print posx(i),posy(-1)," moveto gsave ",textangle," rotate"
        print "(",input[i],") show grestore"
    print "/Times-Roman findfont [",rowwidth,"0 0",rowheight,"0 0] makefont setfont"

    if title:
        print "gsave ", 10, posy(-8), "moveto ( %s ) show grestore" % (title)
        
    for i in range(len(annlist)):
        print "gsave 0 ",posy(i)," translate"
        print "newpath -5 -5 moveto ",annsize-5,0,"lineto",
        print annsize-5,5+rowheight,"lineto",0,5+rowheight,"lineto"
        print "closepath clip (",annlist[i],") 0 0 moveto show grestore"
        # helper lines
        if (greylines > 0 and (i+1)%greylines == 0) or \
           (greylines==0 and i<len(annlist)-1 and not sameannotation(annlist[i],annlist[i+1])):
            print "gsave 0.5 setgray"
            print "0.25 setlinewidth"
            print "[0.25 0.25] 0 setdash"
            print 0,posy(i),"moveto"
            print "(",annlist[i],") stringwidth pop ",annsize," min 0 rmoveto"
            print posx(len(input)),posy(i),"lineto stroke grestore"


def makecolor(color):
    if color == 0.5:
        print "0 0 0"
        return
    color = 1-color
    if color > 0.5:
        #red
        color = min(1,2*color-1)
        col0 = [0.1,0,0.5]
        col1 = [1,0,0]
    else:
        #green
        color = min(1,1-2*color)
        col0 = [0.1,0.2,0.05]
        col1 = [0.9,1,0.2]
    print col0[0]+color*(col1[0]-col0[0]),\
          col0[1]+color*(col1[1]-col0[1]),\
          col0[2]+color*(col1[2]-col0[2])


def makeps(i,j,element):
    (relrep,experp,pexp,ann) = element
    xpos = posx(i)
    if i==-1:
        xpos = 0
    logrelrep = math.log(1.0001 + (relrep / 100.0))
    color = 0.5 + (logrelrep / math.log(4.0))/2  # red=low, green=high
    if experp <= pval0:
        size = maxsize
    elif experp <= pval1:
        size = 0.8*maxsize
    elif experp <= pval2:
        size = 0.6*maxsize
    elif experp <= pval3:
        size = 0.45*maxsize
    elif experp <= pval4:
        size = 0.3*maxsize
    elif experp <= pval5:
        size = 0.2*maxsize
    else:
        size = 0
    if size > 0:
        print xpos,rowheight/2.0 + posy(j),size
        makecolor(color)
        print "makebox"
    else:
        print xpos,rowheight/2.0 + posy(j),1
        makecolor(0.5)
        print "makebox"
    if i==vertical:
        print "gsave 0.5 setgray"
        print "0.25 setlinewidth"
        print "[] 0 setdash"
        print posx(i)-colsize/2,posy(j),"moveto"
        print posx(i)-colsize/2,posy(j+1),"lineto stroke grestore"
    
def makefooter(lineno, footer = ""):
    print "/prtstr {dup stringwidth pop exch gsave 7",posy(lineno),"2 add rmoveto show grestore ",3+colsize," add 0 rmoveto currentpoint translate} def"
    print "/Times-Roman findfont ",colsize," scalefont setfont gsave"
    print "5 0 moveto"
    print "(P values: ) prtstr "
    makeps(-1,lineno,[-99,pval0,0,0])
    print "(<",pval0,") prtstr "
    makeps(-1,lineno,[-99,pval1,0,0])
    print "(<",pval1,") prtstr "
    makeps(-1,lineno,[-99,pval2,0,0])
    print "(<",pval2,") prtstr "
    makeps(-1,lineno,[-99,pval3,0,0])
    print "(<",pval3,") prtstr "
    makeps(-1,lineno,[-99,pval4,0,0])
    print "(<",pval4,") prtstr "
    makeps(-1,lineno,[-99,pval5,0,0])
    print "(<",pval5,") prtstr "
    print "grestore"
    # fold change
    print "gsave 0 ",-colsize,"translate"
    print "5 0 moveto"
    print "(Fold change:) prtstr "
    makeps(-1,lineno,[300,pval0,0,0])
    print "(>+4) prtstr"
    makeps(-1,lineno,[200,pval0,0,0])
    print "(+3) prtstr"
    makeps(-1,lineno,[100,pval0,0,0])
    print "(+2) prtstr"
    makeps(-1,lineno,[50,pval0,0,0])
    print ("(+1.5) prtstr")
    makeps(-1,lineno,[-33,pval0,0,0])
    print ("(-1.5) prtstr")
    makeps(-1,lineno,[-50,pval0,0,0])
    print "(-2) prtstr"
    makeps(-1,lineno,[-66,pval0,0,0])
    print "(-3) prtstr"
    makeps(-1,lineno,[-75,pval0,0,0])
    print "(<-4) prtstr"
    print "grestore"
    if footer:    
        print "gsave ", 0, posy(lineno + 3), "moveto ( %s ) show grestore" % (footer)
        
    
def makepageheader(page):
    print "%%Page: ",page,page
    print "gsave 11 60 translate"


def makepagetrailer(page):
    print "grestore"

def collect( infile, with_headers = False):
    """read input table."""

    data = []

    lines = filter(lambda x: x[0] != "#", infile.readlines())

    if len(lines) == 0: return data
    
    if with_headers:
        del lines[0]
        
    for line in lines:
        if line[0] == "#": continue

        try:
            (code, goid, scount, stotal, spercent, bcount, btotal, bpercent, pover, punder, goid, category, description) = line[:-1].split("\t")[:13]
        except ValueError:
            raise "# parsing error in line: %s" % line[:-1]
            sys.exit(1)
        
        if code == "+":
            p = pover
        else:
            p = punder
            
        data.append( (100.0 * ( float(spercent) / float(bpercent) - 1.0) , abs(float(p)), float(bpercent), description) )
        
    return data

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: go2plot.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-e", "--headers", dest="headers", action="store_true",
                      help="first row is a header [ignored]."  )
    parser.add_option("-t", "--title", dest="title", type="string",
                      help="page title.")
    parser.add_option("-f", "--footer", dest="footer", type="string",
                      help="page footer.")
    parser.add_option("-c", "--column-titles", dest="titles", type="string",
                      help="comma separated list of column titles [default: use filenames]."  )
    parser.add_option("-p", "--pattern-filename", dest="pattern_filename", type="string",
                      help="pattern to map columns to filename."  )

    parser.set_defaults(
        sortAlphabetically = True,
        headers = False,
        titles = "",
        pattern_filename = None,
        title = "",
        footer = "",        
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if len(args) == 0:
        raise "Please supply at least one input file."

    if options.pattern_filename:
        input = []
        titles = args
        for x in titles: input.append( options.pattern_filename % x )
    else:
        input = args
        
        if options.titles:
            titles=options.titles.split(",")
            if len(titles) != len(input):
                raise "Number of titles and files different: %i != %i" % (len(titles), len(input))            
        else:
            titles=input

    if options.loglevel >= 2:
        options.stderr.write( "# reading data from:\n" )
        for x in range(len(input)):
            options.stderr.write( "# %s: %s\n" % ( titles[x], input[x]) )
    
    data = []
    for filename in input:
        # collect all data
        try:
            values = collect( open(filename,"r"), with_headers = options.headers )
        except IOError:
            values = []
        data.append ( values )
        
    # collect all annotations
    # Also filter for max pvalue
    annotations = {}
    for i in range(len(input)):
        for line in data[i]:
            if line[1] < maxp:
                annotations[ line[3] ] = 0

    # Filter for pvalues and relreps in first two columns
    for ann in annotations:
        annotations[ann] = [1.0,1.0,1.0,1.0]

    # sort and filter annotations
    # (Code removed which did some filtering; the annotations data is not used)
    # By removing labels from annlist you can select the annotations you want to display
    annlist = [ ann for ann in annotations ]
    if options.sortAlphabetically:
        annlist.sort()

    # now make table

    # Number of columns
    columns = 1+int(len(annlist)*rowheight / maxheight)
    # Length of one column
    collen = int(maxheight / rowheight)
    # Max number of columns on a single page
    pagecols = int((maxwidth + maincolsep) / (posx(len(titles)) + maincolsep))
    if doEps:
        pagecols = columns
    # Number of pages needed
    numpages = 1 + int((columns-1)/pagecols)

    makeheader( titles, annlist, min(pagecols,columns), numpages )

    for page in range(numpages):
        if page == numpages - 1:
            colsonpage = columns - page*pagecols
        else:
            colsonpage = pagecols

        makepageheader(page+1)

        for column in range(colsonpage):
            globcolumn = page*pagecols + column
            colannlist = annlist[ globcolumn*collen:
                                  min((globcolumn+1)*collen, len(annlist)) ]
            print "gsave"
            print column * ( posx(len(titles)) + maincolsep )," 0 translate"

            makecolheader( titles, colannlist,
                           title = options.title )

            for i in range(len(titles)):
                for j in range(len(colannlist)):
                    # find element
                    element = 0
                    for line in data[i]:
                        if line[3] == colannlist[j]:
                            element = line
                    if element != 0:
                        # show
                        makeps(i,j,element)

            print "grestore"

        if colsonpage > 1:
            makefooter( collen+2, footer = options.footer)
        else:
            makefooter( len(colannlist)+2, footer = options.footer )

        makepagetrailer(page+1)

        if not doEps:
            print "showpage"





    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bam2bidirectionaltranscription.py
=============================================

:Author: Nick Ilott
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

The purpose of this script is two-fold.

1. Stranded RNA-seq libraries are becoming more and more commonplace. As such, a prudent 
   quality control step after alignment to assess the observed strandedness across a set of transcripts of
   interest (GTF file as input).

2. Bidirectional transcription at intergenic loci is an area of interest across biological science.
   This script allows the user to assess the level of bidirectional transcription across a set
   of intervals of interest (GTF file as input).


Usage
-----

Example::

   python bam2bidirectionaltranscription.py --help

Type::

   python bam2bidirectionaltranscription.py --help

for command line help.

Documentation
-------------

Requires both the NH and XS flags to be set in the bam file.
This is specific to the aligner used. TopHat will add an XS and NH
flag for example.

Doesn't use paired information.
 

Code
----

'''

import os
import sys
import re
import optparse
import pysam
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import CGAT.Experiment as E
import CGAT.IndexedGenome as IndexedGenome
import numpy as np
import collections


def filterGtf(gtf_iterable, bamfile, tag_count = 10, length = 200, unique = True):
    '''
    filter the gtf file based on filtering options
    - This does not account for strand at this point
    '''
    filtered_set = set()
    total = 0
    filtered = 0
    for gtf in gtf_iterable:
        total += 1
        tag_number = 0
        if (gtf.end + 1)  - gtf.start < length:
            filtered += 1
            continue
        start, end = gtf.start-1, gtf.end
        for alignment in bamfile.fetch(gtf.contig, start, end):
            # get the number of hits
            nh = [x for x in alignment.tags if x[0] == "NH"]
            nh = nh[0][1]
            if unique:
                if nh != 1: continue
            tag_number += 1
        if tag_number < tag_count: 
            filtered += 1
            continue
        filtered_set.add(gtf.gene_id)
    E.info("total gtf entries in = %i...total filtered = %i" % (total, filtered))
    return filtered_set


def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = optparse.OptionParser( version = "%prog version: $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )
    parser.add_option("-b", "--bam", dest="bam", type="string",
                      help="input bam file"  )
    parser.add_option("-a", "--annotation", dest="annotation", type="string",
                      help="input annotation gtf file"  )
    parser.add_option("-m", "--merge-transcripts", dest="merge_transcripts", action="store_true"
                      , help="merge transcripts with the same gene id"  )
    parser.add_option("-u", "--unique", dest="unique", action="store_true"
                      , help="only use unique alignments"  )
    parser.add_option("-t", "--tag-count", dest="tag_count", type="int"
                      , help="minimum tag count to allow inclusion in analysis"  )
    parser.add_option("-l", "--length", dest="tag_count", type="int"
                      , help="minimum length"  )
    parser.add_option("-n", "--analysis-type", dest = "analysis_type", type = "choice", choices = ("ratio", "profile", "shape"))
    parser.add_option("-o", "--outbase", dest="outbase", type = "string"
                      , help="basename for outfiles"  )
    parser.add_option("-s", "--bin-number", dest="bin_number", type = "int"
                      , help="number of bins to use - only active with --profile"  )

    parser.set_defaults(unique = True
                        , merge_transcripts = True
                        , tag_count = 10
                        , length = 200
                        , analysis_type = "ratio"
                        , bin_number = 100
                        , outbase = "bidirection"
                        , output_gtf = False)
                        

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    bam = pysam.Samfile(options.bam)
    gtffile = IOTools.openFile(options.annotation)

    if options.merge_transcripts:
        iterable = GTF.merged_gene_iterator(GTF.iterator(gtffile))
    else:
        iterable = GTF.iterator(gtffile)

    E.info("filtering gtf set")
    gtf_set = filterGtf(iterable, bam, tag_count = options.tag_count, length = options.length, unique = options.unique)

    # reinitiate iterable
    gtffile = IOTools.openFile(options.annotation)
    if options.merge_transcripts:
        iterable = GTF.merged_gene_iterator(GTF.iterator(gtffile))
    else:
        iterable = GTF.iterator(gtffile)
    
    ################################################
    # output the overall ratio of sense to antisense
    # reads for the gtf entries filtered in gtf_set
    ################################################
    if options.analysis_type == "ratio":
        E.info("calculating ratio of sense / antisense")
        outf = open(options.outbase + "_ratio.tsv", "w")
        outf.write("gene_id\ttranscript_id\tasratio\n")
        total_unmapped = 0
        total_no_xs = 0
        total_reads_in = 0
        for gtf in iterable:
            if gtf.gene_id not in gtf_set: continue
            transcription = {"sense": 0, "antisense": 0}
            start, end = gtf.start-1, gtf.end
            for alignment in bam.fetch(gtf.contig, start, end):
                total_reads_in += 1
                if alignment.is_unmapped:
                    total_unmapped += 1
                    continue
                # get the strand origin
                xs = [x for x in alignment.tags if x[0] == "XS"]
                if len(xs) == 0:
                    E.warn("No XS tag present, read %s not included in the analysis" % alignment.qname)
                    total_no_xs += 1
                    continue
                xs = xs[0][1]
                if xs == "+":
                    transcription["sense"] += 1
                elif xs == "-":
                    transcription["antisense"] += 1
                else:
                    raise ValueError("no such strand %s" % xs)
            if transcription["sense"] == 0 and transcription["antisense"] == 0:
                continue
            elif transcription["sense"] == 0 and transcription["antisense"] != 0:
                ratio = "-Inf"
            elif transcription["sense"] != 0 and transcription["antisense"] == 0:
                ratio = "Inf"
            else:
                gtf_set.add(gtf.gene_id)
                ratio = float(transcription["sense"]) / transcription["antisense"]
            outf.write("%s\t%s\t%s\n" % (gtf.gene_id, gtf.transcript_id, str(ratio)))
        outf.close()

    ####################################################
    # profiling average number of reads across intervals
    ####################################################
    elif options.analysis_type == "profile":
        profile = {}
        E.info("assessing profile over intervals in %i bins" % options.bin_number)
        total_unmapped = 0
        total_no_xs = 0
        total_reads_in = 0
        for gtf in iterable:
            if gtf.gene_id not in gtf_set: continue
            start, end = gtf.start-1, gtf.end
            bins = np.histogram(range(start, end), bins = options.bin_number)[1]
            bin_as = []
            for x in range(len(bins)):
                if x < len(bins)-1:
                    bin_size = int(bins[x+1]) - int(bins[x])
                    c_sense = 0
                    c_antisense = 0
                    for alignment in bam.fetch(gtf.contig, int(bins[x]), int(bins[x+1])):
                        total_reads_in += 1
                        if alignment.is_unmapped:
                            total_unmapped += 1
                            continue
                        # number of hits
                        nh = [x for x in alignment.tags if x[0] == "NH"]
                        nh = nh[0][1]
                        if options.unique:
                            if nh != 1: continue
                        # get the strand origin
                        xs = [x for x in alignment.tags if x[0] == "XS"]
                        if len(xs) == 0: 
                            E.warn("No XS tag present, read %s not included in the analysis" % alignment.qname)
                            total_no_xs += 1
                            continue
                        xs = xs[0][1]
                        if xs == "+":
                            c_sense += 1
                        elif xs == "-":
                            c_antisense += 1
                        else:
                            raise ValueError("no such strand %s" % xs)
                    if c_sense == 0 and c_antisense == 0:
                        # if bin is empty then add "NA"
                        ratio = "NA"
                    elif c_sense == 0 and c_antisense != 0:
                        ratio = "-Inf"
                    elif c_sense != 0 and c_antisense == 0:
                        ratio = "Inf"
                    else:
                        ratio = float(c_sense)/c_antisense
                    bin_as.append(ratio)
            profile[gtf.gene_id] = bin_as

        outf = open(options.outbase + "_profile.tsv", "w")
        outf.write("\t".join(profile.keys()) + "\n")
        for x in zip(*profile.values()):
            outf.write("\t".join(map(str,list(x))) + "\n")
        outf.close()

    ####################################################
    # provide table with sense and antisense
    # counts
    ####################################################
    elif options.analysis_type == "shape":
        sense = {}
        antisense = {}
        E.info("assessing shape over intervals in %i bins" % options.bin_number)
        total_unmapped = 0
        total_no_xs = 0 
        total_reads_in = 0
        for gtf in iterable:
            if gtf.gene_id not in gtf_set: continue
            start, end = gtf.start-1, gtf.end
            bins = np.histogram(range(start, end), bins = options.bin_number)[1]
            bin_as = []
            bin_s = []
            for x in range(len(bins)):
                if x < len(bins)-1:
                    bin_size = int(bins[x+1]) - int(bins[x])
                    c_sense = 0
                    c_antisense = 0
                    for alignment in bam.fetch(gtf.contig, int(bins[x]), int(bins[x+1])):
                        total_reads_in += 1
                        if alignment.is_unmapped:
                            total_unmapped += 1
                            continue
                        # number of hist
                        nh = [x for x in alignment.tags if x[0] == "NH"]
                        nh = nh[0][1]
                        if options.unique:
                            if nh != 1: continue
                        # get the strand origin
                        xs = [x for x in alignment.tags if x[0] == "XS"]
                        if len(xs) == 0: 
                            E.warn("No XS tag present, read %s not included in the analysis" % alignment.qname)
                            total_no_xs += 1
                            continue
                        xs = xs[0][1]
                        if xs == "+":
                            c_sense += 1
                        elif xs == "-":
                            c_antisense += 1
                        else:
                            raise ValueError("no such strand %s" % xs)
                    bin_s.append(c_sense)
                    bin_as.append(c_antisense)
            sense[gtf.gene_id] = bin_s
            antisense[gtf.gene_id] = bin_as

        outf = open(options.outbase + "_shape.tsv", "w")
        outf.write("\t".join(sense.keys()) + "\tstatus" + "\n")
        for x in zip(*sense.values()):
            outf.write("\t".join(map(str,list(x))) + "\tsense" + "\n")
        for x in zip(*antisense.values()):
            outf.write("\t".join(map(str,list(x))) + "\tantisense" + "\n")
        outf.close()

    E.info("number of reads in: %i" % total_reads_in)
    E.info("number of unmapped reads in bam file: %i" % total_unmapped)
    E.info("number of reads with no XS tag %i" % total_no_xs)

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#   Gene prediction pipeline 
#
#   $Id: tree_strain2species.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
trees_train2species.py - convert trees containing strains to species trees
===========================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a collection of trees from stdin and collapses all nodes that
contain only children of the same species but different strains.

This script was used for the yeast project.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

""" 

import os
import sys
import string
import re
import optparse
import subprocess

import scipy.stats

import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools
import CGAT.IOTools as IOTools

def parseIdentifier( id, options ):

    data = id.split(options.separator)
    if len(data) == 4:
        return data
    elif len(data) == 2:
        s,g = data
        t = g
        q = "UK"
        return s,t,g,q

def buildIdentifier( schema, transcript, gene, quality, options ):

    if transcript == None:
        return options.separator.join( (schema, gene ) )
    else:
        return options.separator.join( (schema, transcript, gene, quality ) )

def getMergers( tree, map_strain2species, options):
    """merge strains to species.

    returns the new tree with species merged and
    a dictionary of genes including the genes that have been merged.

    Currently, only binary merges are supported.
    """
    
    n = TreeTools.GetSize( tree ) + 1
    all_strains = map_strain2species.keys()
    all_species = map_strain2species.values()
    
    genes = []
    for x in range(n):
        g = {}
        for s in all_strains: g[s] = set()
        genes.append( g )
        
    ## build list of species pairs that can be joined.
    map_species2strain = IOTools.getInvertedDictionary( map_strain2species )
    pairs = []

    for species, strains in map_species2strain.items():
        for x in range(len(strains)):
            for y in range(0,x):
                pairs.append( (strains[x], strains[y]) )

    ## map of genes to new genes
    ## each entry in the list is a pair of genes of the same species
    ## but different strains to be joined.
    map_genes2new_genes = []

    ## dictionary of merged genes. This is to ensure that no gene
    ## is merged twice
    merged_genes = {}

    def count_genes( node_id ):
        """record number of genes per species for each node

        This is done separately for each strain. The counts are aggregated for each species
        over strains by taking the maximum gene count per strain. This ignores any finer
        tree structure below a species node.
        """
        node = tree.node(node_id)
        
        if node.succ:
            this_node_set = genes[node_id]
            ## process non-leaf node
            for s in node.succ:

                ## propagate: terminated nodes force upper nodes to terminate (assigned to None).
                if not genes[s]:
                    this_node_set = None
                    break

                ## check if node merges genes that are not part of the positive set
                for strain in all_strains:
                    if strain in map_strain2species:
                        # merge genes from all children
                        this_node_set[strain] = this_node_set[strain].union(genes[s][strain])
                        
                        if len(this_node_set[strain]) > 1:
                            # more than two genes for a single species, so no join
                            this_node_set = None
                            break
                        
                    elif strain not in map_strain2species and \
                           this_node_set[strain] > 0:
                        this_node_set = None
                        break

            if this_node_set == None:
                genes[node_id] = None
                return

            for strain_x, strain_y in pairs:
                if len(this_node_set[strain_x]) == 1 and len(this_node_set[strain_y]) == 1:
                    species = map_strain2species[strain_x]
                    gene_x, gene_y = tuple(this_node_set[strain_x])[0], tuple(this_node_set[strain_y])[0]

                    ## check if these to genes have already been merged or are
                    ## merged with other partners already
                    ## The merged genes are assigned the same node_id, if they have
                    ## been already merged.
                    key1 = strain_x + gene_x
                    key2 = strain_y + gene_y
                    if key1 > key2: key1, key2 = key2, key1
                    
                    merge = False
                    if key1 in merged_genes and key2 in merged_genes:
                        if merged_genes[key1] == merged_genes[key2]:
                            merge = True
                    elif key1 not in merged_genes and key2 not in merged_genes:
                        merge = True
                        merged_genes[key1] = node_id
                        merged_genes[key2] = node_id
                        
                    if merge:
                        map_genes2new_genes.append( (node_id, species, strain_x, gene_x, strain_y, gene_y ) )
                        
                    ## once two genes have been joined, they can not be remapped further
                    genes[node_id] = None
                    return
        else:
            ## process leaf
            strain, t, g, q = parseIdentifier( node.data.taxon, options )

            if strain in map_strain2species:
                genes[node_id][strain].add( g )
            else:
                ## do not process nodes that do not need to be mapped
                genes[node_id] = None
    
    tree.dfs( tree.root, post_function = count_genes )

    return map_genes2new_genes

def applyMergers( tree, mergers, counters, map_strain2species, options ):
    """apply mergers to a tree."""
    
    new_genes = {}
    for node_id, species, strain_x, gene_x, strain_y, gene_y in mergers:
        if species not in counters:
            counters[species] = 0
        else:
            counters[species] += 1
            
        new_name = buildIdentifier( species, None, options.pattern_gene % counters[species], None, options )
        tree.truncate( node_id, new_name )
        new_genes[new_name] = [ (strain_x, gene_x), (strain_y, gene_y) ]

    ## rename all remaining taxa
    for n in tree.get_terminals():
        strain, t, g, q = parseIdentifier( tree.node(n).data.taxon, options )

        if strain in map_strain2species:
            species = map_strain2species[strain]
            if options.keep_old_names:
                new_name =  buildIdentifier( species, t, g, q)
            else:
                if species not in counters:
                    counters[species] = 0
                else:
                    counters[species] += 1
                new_name = buildIdentifier( species, None, options.pattern_gene % counters[species], None, options )

            tree.node(n).data.taxon = new_name
            new_genes[new_name] = [(strain, g),]
            
    return new_genes

def processGeneTrees( chunks, lines, map_strain2species, options ):
    """process gene trees."""

    if options.output_filename_genes:
        output_genes = open(options.output_filename_genes, "w" )
    else:
        output_genes = options.stdout

    ## for counting genes
    counters = {}
    ## dictionary of merged genes, used to test if some genes appear more than once
    merged = {}
    
    def processChunk( lines, map_strain2species, options ):

        nexus = TreeTools.Newick2Nexus( lines )
        global ninput, noutput, nskipped, nmerged
        
        for tree in nexus.trees:
            ninput += 1

            if options.loglevel >= 3: tree.display()

            mergers = getMergers( tree, map_strain2species, options )

            if options.loglevel >= 3:
                options.stdlog.write("# found %i pairs of genes that will be merged.\n" % (len(mergers) ) )
            
            if len(mergers) > 0:
                nmerged += 1

            n = applyMergers( tree, mergers, counters, map_strain2species, options )

            if len(tree.get_terminals()) <= 1:
                nskipped += 1
                continue

            for new_name, values in n.items():
                for strain, gene in values:
                    if (strain,gene) in merged:
                        options.stdlog.write( "# warning: strain %s and gene %s already appeared in tree %s" % (merged[(strain,gene)]))
                        nwarnings += 1
                    merged[(strain,gene)] = None
                    output_genes.write( "%s\t%s\n" % (options.separator.join((strain,gene)), new_name ) )

            tree.writeToFile( options.stdout, format = options.output_format )
            noutput += 1
            
    if chunks:
        for c in range(len(chunks)-1):
            a, b = chunks[c], chunks[c+1]
            processChunk( lines[a:b], map_strain2species, options )
    else:
        processChunk( lines, map_strain2species, options )

    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, nskipped=%i, nmerged=%i\n" % (ninput, noutput, nskipped, nmerged))

def getSpeciesTreeMergers( tree, full_map_strain2species, options ):
    """merge strains to species.

    Simply rename all taxa of strains to the species.
    """
    
    nnodes = TreeTools.GetSize( tree ) + 1

    map_strain2species = {}
    for n in tree.get_terminals():
        node = tree.node(n)
        taxon = node.data.taxon
        if taxon in full_map_strain2species:
            map_strain2species[taxon] = full_map_strain2species[taxon]
            node.data.taxon = map_strain2species[taxon]
            
    if len(map_strain2species) == 0: return []

    all_species = tree.get_taxa()
    mapped_species = set(map_strain2species.values())

    species_at_node = []
    for x in range(nnodes):
        g = {}
        for s in all_species: g[s] = 0
        species_at_node.append( g )
        
    def count_species( node_id ):
        """record species for each node
        """
        node = tree.node(node_id)
        if node.succ:
            ## process non-leaf node
            for s in node.succ:
                for species in all_species:
                    species_at_node[node_id][species] += species_at_node[s][species]
        else:
            ## process leaf
            species = node.data.taxon
            species_at_node[node_id][species] = 1
    
    tree.dfs( tree.root, post_function = count_species )
    
    # now merge all those that contain only a single species
    # proceed top-down
    
    nodes_to_skip = set()
    mergers = []

    def merge_species( node_id ):
        
        if node_id in nodes_to_skip: return

        total = sum( species_at_node[node_id].values() )
        for species in mapped_species:
            if species_at_node[node_id][species] <= 1 or \
                    species_at_node[node_id][species] != total: continue
            
            ## merge species
            children = tree.get_leaves( node_id )
            for child in children:
                nodes_to_skip.add( child)

            mergers.append( (node_id, children) )
                
    tree.dfs( tree.root, pre_function = merge_species )

    return mergers

def applySpeciesTreeMergers( tree, mergers, map_strain2species, options ):
    """apply mergers to a tree."""

    for node_id, children in mergers:

        node = tree.node(node_id)

        branch_lengths = [ tree.node(c).data.branchlength for c in children ]

        # copy taxon name from first child
        node.data.taxon = tree.node(children[0]).data.taxon

        # set new branch length
        if options.merge_mode == "ignore":
            pass
        elif options.merge_mode == "add-mean":
            node.data.branchlength += scipy.mean( branch_lengths )
        elif options.merge_mode == "add-max":
            node.data.branchlength += max( branch_lengths )
        elif options.merge_mode == "add-min":
            node.data.branchlength += min( branch_lengths )
            
        # remove all children
        for child in children:
            tree.truncate( child, keep_node = node_id )

def processSpeciesTrees( chunks, lines, map_strain2species, options ):
    """process gene trees."""

    def processChunk( lines, map_strain2species, options ):

        nexus = TreeTools.Newick2Nexus( lines )
        global ninput, noutput, nskipped, nmerged
        
        for tree in nexus.trees:
            ninput += 1

            if options.loglevel >= 3: tree.display()

            mergers = getSpeciesTreeMergers( tree, map_strain2species, options )

            if options.loglevel >= 3:
                options.stdlog.write("# found %i nodes in the tree that will be merged.\n" % (len(mergers) ) )
            
            if len(mergers) > 0:
                nmerged += 1

            n = applySpeciesTreeMergers( tree, mergers, map_strain2species, options )

            if len(tree.get_terminals()) <= 1:
                nskipped += 1
                continue

            tree.writeToFile( options.stdout, format = options.output_format )
            noutput += 1
            
    if chunks:
        for c in range(len(chunks)-1):
            a, b = chunks[c], chunks[c+1]
            processChunk( lines[a:b], map_strain2species, options )
    else:
        processChunk( lines, map_strain2species, options )

    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, nskipped=%i, nmerged=%i\n" % (ninput, noutput, nskipped, nmerged))

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: tree_strain2species.py 2782 2009-09-10 11:40:29Z andreas $",
                                    usage = globals()["__doc__"] )

    parser.add_option( "--filename-synonyms", dest="filename_synonyms", type="string" ,
                       help="filename with synonyms. Use this to aggregate several strains for a species.")

    parser.add_option( "--filename-genes", dest="output_filename_genes", type="string" ,
                       help="output filename with new gene names.")

    parser.add_option( "--species-tree", dest="species_tree", action="store_true",
                       help="input tree are species trees. If not given, the trees are assumed to be gene trees." )
                       
    parser.add_option( "--merge-mode", dest="merge_mode", type="choice",
                       choices=("ignore", "add-mean", "add-max", "add-min" ),
                       help="how to deal with branch lengths of merged nodes." )
    
    parser.set_defaults(
        filename_synonyms = "map_strain2species",
        pattern_gene = "J%06i",
        output_format = "nh", 
        separator = "|",
        output_filename_genes = None,
        keep_old_names = False,
        species_tree = False,
        merge_mode = "ignore",
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    ########################################################################
    ########################################################################
    ########################################################################
    ## read synonyms
    if options.filename_synonyms:
        infile = open(options.filename_synonyms, "r" )
        map_strain2species = IOTools.ReadMap( infile )
        infile.close()
    else:
        map_strain2species = {}

    lines = map(lambda x: x[:-1], sys.stdin.readlines())

    ninput, noutput, nskipped, nmerged = 0, 0, 0, 0

    ## iterate over chunks
    chunks = filter( lambda x: lines[x][0] == ">", range(len(lines)))
    if len(chunks) == 0: chunks=[0]
    chunks.append(len(lines))

    if options.species_tree:
        processSpeciesTrees( chunks, lines, map_strain2species, options )
    else:
        processGeneTrees( chunks, lines, map_strain2species, options )

    E.Stop()



#!/usr/bin/env python

"""
wiggle_build_index.py - build an index for a wiggle track
=========================================================

If index_file is not provided wiggle_file.index is used.

usage: %prog wiggle_file index_file
--version: version string

"""

import psyco_full

from bx.cookbook import doc_optparse
from bx import interval_index_file
import bx.wiggle

import sys
import os.path
from bx.misc.seekbzip2 import SeekableBzip2File

def main():

    # Parse command line

    options, args = doc_optparse.parse( __doc__ )
    if options.version: return

    try:
        wiggle_file = args[0]
        # If it appears to be a bz2 file, attempt to open with table
        if wiggle_file.endswith( ".bz2" ):
            table_file = wiggle_file + "t"
            if not os.path.exists( table_file ):
                doc_optparse.exit( "To index bz2 compressed files first "
                                   "create a bz2t file with bzip-table." )
            # Open with SeekableBzip2File so we have tell support
            wiggle_in = SeekableBzip2File( wiggle_file, table_file )
            # Strip .bz2 from the filename before adding ".index"
            wiggle_file = wiggle_file[:-4]
        elif wiggle_file.endswith( ".lzo" ):
            from bx.misc.seeklzop import SeekableLzopFile
            table_file = wiggle_file + "t"
            if not os.path.exists( table_file ):
                doc_optparse.exit( "To index lzo compressed files first "
                                   "create a lzot file with bzip-table." )
            # Open with SeekableBzip2File so we have tell support
            wiggle_in = SeekableLzopFile( wiggle_file, table_file )
            # Strip .lzo from the filename before adding ".index"
            wiggle_file = wiggle_file[:-4]
        else:
            wiggle_in = open( wiggle_file )
        # Determine the name of the index file
        if len( args ) > 1:
            index_file = args[1]
        else:
            index_file = wiggle_file + ".index"
    except:
        doc_optparse.exception()

    indexes = interval_index_file.Indexes()

    # Can't use the iterator, as there is no next() and thus
    # no way to access the positions. The following code is 
    # modified from wiggle.py
    last_chrom = None
    start = None
    end = None
    first_pos = None

    # always for wiggle data
    strand = '+'

    mode = "bed"

    while 1:
        pos = wiggle_in.tell()
        line = wiggle_in.readline()
        if not line: break

        if line.isspace() or line.startswith( "track" ) or line.startswith( "#" ) or line.startswith( "browser" ):
            continue
        elif line.startswith( "bed" ):
            indexes.add( fields[0], int( fields[1] ), int( fields[2] ), pos )
        elif line.startswith( "variableStep" ) or line.startswith( "fixedStep"):
            if first_pos != None:
                indexes.add( last_chrom, start, end, first_pos )
            first_pos = pos
            header = bx.wiggle.parse_header( line )
            last_chrom = header['chrom']
            start = int(header['start']) - 1
            end = start
            current_step = None
            if 'span' in header: 
                current_span = int( header['span'] )
            else: 
                current_span = 1
            if 'step' in header: 
                current_step = int( header['step'] )

            if line.startswith( "variableStep" ):
                mode = "variableStep"
            else:
                mode = "fixedStep"
        elif mode == "variableStep":
            fields = line.split()
            end = int( fields[0] ) - 1 + current_span
        elif mode == "fixedStep":
            end += current_step
        else:
            raise "Unexpected input line: %s" % line.strip()

    out = open( index_file, 'w' )
    indexes.write( out )
    out.close()

if __name__ == "__main__": main()





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
split_fasta.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python split_fasta.py --help

Type::

   python split_fasta.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import os
import optparse
import CGAT.FastaIterator as FastaIterator
import CGAT.IOTools as IOTools
import CGAT.Experiment as E

class Files:

    mFiles = {}
    
    def __init__(self,
                 output_pattern   = None,
                 skip_identifiers = False):

        self.mOutputPattern = output_pattern
        self.mSkipIdentifiers = skip_identifiers
        self.mCounts = {}
        
    def __del__(self):
        """close all open files."""
        for file in self.mFiles.values():
            file.close()

    def GetFile( self, identifier ):
        return identifier

    def GetFilename( self, identifier ):
        """get filename for an identifier."""

        if self.mOutputPattern:
            return re.sub( "%s", str(identifier), self.mOutputPattern )
        else:
            return identifier

    def OpenFile( self, filename, mode = "w" ):
        """open file.
        
        If file is in a new directory, create directories.
        """
        if mode in ("w", "a"):
            dirname = os.path.dirname(filename)
            if dirname and not os.path.exists( dirname ):
                os.makedirs( dirname )
                
        return open( filename, mode)
        
    def Write( self, identifier, sequence ):

        filename = files.GetFilename( identifier )
        
        if filename not in self.mFiles:
            
            if len(self.mFiles) > 1000:
                for f in self.mFiles.values(): f.close()
                self.mFiles = {}
                
            self.mFiles[filename] = self.OpenFile( filename, "a" )

        if self.mSkipIdentifiers:
            self.mFiles[filename].write( "%s\n" % (sequence.sequence))
        else:
            self.mFiles[filename].write( ">%s\n%s\n" % (sequence.title, sequence.sequence))

        if filename not in self.mCounts:
            self.mCounts[filename] = 0
        self.mCounts[ filename ] += 1

    def DeleteFiles( self, min_size = 0 ):
        """delete all files below a minimum size."""

        ndeleted = 0
        for filename, counts in self.mCounts.items():
            if counts < min_size:
                os.remove( filename )
                ndeleted += 1
                
        return ndeleted

class FilesChunks(Files):

    def __init__(self,
                 chunk_size, **kwargs):

        Files.__init__(self, **kwargs )
        self.mChunkSize = chunk_size
        self.mFilename = 0
        
    def GetFilename(self, identifier):

        if not self.mFilename or self.mCounts[self.mFilename] % self.mChunkSize == 0:
            self.mFilename = re.sub( "%s", str(len(self.mCounts) + 1), self.mOutputPattern )
            
        return self.mFilename
        
            
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: split_fasta.py 1714 2007-12-11 16:51:12Z andreas $")
    
    parser.add_option("-f", "--file", dest="input_filename", type="string",
                      help="input filename. If not given, stdin is used.",
                      metavar="FILE" )

    parser.add_option ("-i", "--input-pattern", dest="input_pattern", type="string",
                       help="input pattern. Parses description line in order to extract id." )

    parser.add_option ("-o", "--output-pattern", dest="output_pattern", type="string",
                       help="output pattern. Gives filename for a given sequence." )

    parser.add_option ("-n", "--num-sequences", dest="num_sequences", type="int",
                       help="split by number of sequences (not implemented yet)." )

    parser.add_option ("-m", "--map", dest="map_filename", type="string",
                       help="map filename. Map identifiers to filenames",
                       metavar="FILE" )

    parser.add_option ("-s", "--skip-identifiers", dest="skip_identifiers", action="store_true",
                       help="do not write identifiers.",
                       metavar="FILE" )

    parser.add_option ("--min-size", dest="min_size", type="int",
                       help="minimum cluster size.")

    parser.set_defaults( \
        input_filename = None,
        map_filename = None,
        skip_identifiers = False,
        input_pattern = "^(\S+)",
        min_size = 0,
        num_sequences = None,
        output_pattern = "%s" )

    (options, args) = E.Start( parser ) 

    if options.input_filename:
        infile = IOTools.openFile( options.input_filename, "r")
    else:
        infile = sys.stdin

    if options.map_filename:
        map_id2filename = IOTools.ReadMap( open( options.map_filename, "r") )
    else:
        map_id2filename = {}
        
    if options.num_sequences:
        files = FilesChunks( chunk_size = options.num_sequences,
                             output_pattern = options.output_pattern,
                             skip_identifiers = options.skip_identifiers )
        
    else:
        files = Files( output_pattern = options.output_pattern,
                       skip_identifiers = options.skip_identifiers )

    if options.input_pattern:
        rx = re.compile( options.input_pattern )
    else:
        rx = None

    ninput = 0
    noutput = 0
    identifier = None
    chunk = 0
    
    for seq in FastaIterator.iterate( infile ):
        
        ninput += 1

        if rx:
            try:
                identifier = rx.search(seq.title).groups()[0]
            except AttributeError:
                print "# parsing error in description line %s" % (seq.title)
        else:
            identifier = seq.title

        if map_id2filename:
            if identifier in map_id2filename:
                identifier = map_id2filename[identifier]
            else:
                continue

        files.Write( identifier, seq )
        noutput += 1

    if options.input_filename:
        infile.close()

    ## delete all clusters below a minimum size
    ## Note: this has to be done at the end, because
    ## clusters sizes are only available once both the fasta
    ## file and the map has been parsed.
    if options.min_size:
        ndeleted = files.DeleteFiles( min_size = options.min_size )
    else:
        ndeleted = 0
        
    if options.loglevel >= 1:
        print "# input=%i, output=%i, ndeleted=%i" % (ninput, noutput, ndeleted)
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
simulate_function.py - create datapoints for a function
=======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python simulate_function.py --help

Type::

   python simulate_function.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time
import optparse

import CGAT.Experiment as E
from scipy.stats import *

USAGE="""simulate_function.py [options] [infile] < stdin

Create data points for a function. All the scipy stats function
are imported.

Examples:

--function="norm,cdf( x, loc=0.0150, scale=0.1225 )" --xrange=-3,3,0.01
"""

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: simulate_function.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-x", "--xrange", dest="xrange", type="string",
                      help="xrange."  )
    parser.add_option("-y", "--yrange", dest="yrange", type="string",
                      help="yrange."  )
    parser.add_option("-f", "--function", dest="function", type="string",
                      help="function."  )


    parser.set_defaults(
        xtitle = "x",
        ytitle = "y",
        xrange = "0,1,0.1",
        yrange = None,
        function = "x",
        value_format = "%6.4f" )

    (options, args) = E.Start( parser )

    if options.xrange: options.xrange = map(float, options.xrange.split(","))
    if options.yrange: options.yrange = map(float, options.yrange.split(","))

    exec("f = lambda x: %s" % options.function )
        
    options.stdout.write( "x\ty\n" )
    x = options.xrange[0]
    while x <= options.xrange[1]:
        options.stdout.write( "\t".join( map( lambda x: options.value_format % x, 
                                              (x, f(x)) )) + "\n" )
        x += options.xrange[2]

    E.Stop()
    
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
calculate_histogram_2D.py - calculate 2D histogram
==================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script computes a 2D histogram. The data is a tab-separated
with ``x``, ``y`` in the first two columns. The columns can be
changed by the ``-1`` and ``-2`` options.

Usage
-----

Example::

   python calculate_histogram_2D.py --help

Type::

   python calculate_histogram_2D.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time
import optparse

import CGAT.Experiment as E
import CGAT.Histogram2D as Histogram2D

USAGE = """python calculate_histogram.py < stdin > stdout

-1, --column1    column to take [default = 0]
-2, --column2    column to take [default = 1]
# at start of line is a comment
"""

param_column1 = 0
param_column2 = 1
param_bin_size1 = 1
param_bin_size2 = 1
param_titles = True

##---------------------------------------------------------------------------------------------------------        
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: calculate_histogram_2D.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-t", "--titles", dest="titles", action="store_true",
                      help="input data has title in first row [default=%default]." )

    parser.add_option( "--no-titles", dest="titles", action="store_false",
                      help="input data has no title in first row [default=%default]." )

    parser.add_option("-1", "--column1", dest="column1", type="int",
                      help="first column to use [default=%default]." )
 
    parser.add_option("-2", "--column2", dest="column2", type="int",
                      help="second column to use [default=%default]." )
 
    parser.add_option("--bin-size1", dest="bin_size1", type="float",
                      help="bin size for first column [default=%default]." )

    parser.add_option("--bin-size2", dest="bin_size2", type="float",
                      help="bin size for second column [default=%default]." )

    parser.set_defaults( 
        column1 = 1,
        column2 = 2,
        bin_size1 = 1.0,
        bin_size2 = 1.0,
        titles = True )


    (options, args) = E.Start( parser )                                  
    options.column1 -= 1
    options.column2 -= 1

    histograms = []

    vals = []
    
    # retrieve histogram
    lines = filter( lambda x: x[0] <> "#", sys.stdin.readlines())

    if options.titles:
        data = string.split(lines[0][:-1], "\t")        
        print "\t".join((data[options.column1], data[options.column2], "counts"))
        del lines[0]

    ninput, noutput, nskipped = 0, 0, 0

    for l in lines:
        ninput += 1
        data = string.split(l[:-1], "\t")

        try:
            val = map( string.atof, (data[options.column1],data[options.column2]))
        except IndexError:
            nskipped += 1
            continue
        except ValueError:
            nskipped += 1
            continue

        vals.append( val )
        noutput += 1

    lines = None
    
    h = Histogram2D.Calculate( vals, bin_function=lambda x:(int(x[0] / options.bin_size1), int(x[1] / options.bin_size2)) )

    Histogram2D.Print(h, bin_function=lambda x: (x[0] * options.bin_size1, x[1] * options.bin_size2, x[2]))

    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, nskipped=%i\n" % (ninput,noutput,nskipped) )

    E.Stop()




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
r_test.py - t-test and wilcoxon tests
=====================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Interface to R to run tests on a vector of values.

Usage
-----

Example::

   python r_test.py --help

Type::

   python r_test.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import time
import random
import rpy2
from rpy2.robjects import r as R
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Stats as Stats

if __name__  == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: r_test.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option( "-m", "--method", dest="method", type="choice",
                       help="method to use [t-test=t-test,wilcox=wilcox]",
                       choices=( "t-test", "wilcox") )
    parser.add_option( "-1", "--infile", dest="filename_input", type="string" ,
                       help="input filename with vector of values.")
    parser.add_option( "-2", "--infile2", dest="filename_input2", type="string" ,
                       help="input filename with vector of values.")
    parser.add_option( "--header", dest="header", type="string",
                       help="""header of value column [default=%default].""")
                       

    parser.set_defaults(
        method = "t-test",
        filename_input = None,
        header = "value",
        )
    
    (options, args) = E.Start( parser,
                               add_pipe_options = True )


    if options.filename_input:
        infile = open(options.filename_input, "r")
    else:
        infile = sys.stdin

    values, errors = IOTools.ReadList( infile,
                                       map_function=float )
    if options.filename_input:
        infile.close()

    if errors:
        E.warn("errors in input: %s" % ";".join(map(str, errors)))

    kwargs = {}
    xargs = []
    for arg in args:
        if "=" in arg:
            key,value = arg.split("=")
            kwargs[key] = value
        else:
            xargs.append( arg )

    if options.filename_input2:
        infile = open(options.filename_input2, "r")
        values2, errors2 = IOTools.ReadList( infile,
                                             map_function=float )
        infile.close()
    else:
        values2 = None

    stat = Stats.Summary(values)

    power, diff_at_power95 = None, None
    if options.method == "t-test":
        if values2:
            result = R.t_test( values, values2, *xargs, **kwargs )
        else:
            result = R.t_test( values, *xargs, **kwargs )
            ## compute power of test
            power = R.power_t_test(n=len(values), 
                                         delta=abs(stat["mean"]), 
                                         sd=stat["stddev"],
                                         sig_level=0.05 )['power']
            diff_at_power95= R.power_t_test(n=len(values), 
                                            power=0.95,
                                            sd=stat["stddev"],
                                            sig_level=0.05 )['delta']


    if options.method == "wilcox":
        result = R.wilcox_test( values, *xargs, **kwargs )
        
    options.stdout.write( "%s\t%s\n" % ("key", options.header) )

    for key, value in sorted(result.items()):
        if key == "data.name": continue
        if key == "p.value":
            options.stdout.write( "%s\t%5.2e\n" % (str(key), value) )
        else:
            options.stdout.write( "%s\t%s\n" % (str(key), str(value)))
        
    for key, value in stat.items():
        options.stdout.write( "%s\t%s\n" % (str(key), str(value)))

    if power:
        options.stdout.write( "1-power\t%5.2e\n" % (1.0 - power ))
        options.stdout.write( "diff_at_power95\t%f\n" % diff_at_power95 )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
fasta2variants.py - create sequence variants from a set of sequences
====================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a collection of sequences in :term:`fasta` format and
outputs a table of possible variants. It outputs for each position in
a protein sequence the number of variants. 

If the input sequences are cds sequences, for each variant a weight
is output indicating the number of times that variant can occur
from single nucleotide changes.

Usage
-----

Example::

   python fasta2variants.py

Type::

   python fasta2variants.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse
import math
import collections

import CGAT.Experiment as E
import CGAT.Genomics as Genomics
import CGAT.IOTools as IOTools
import CGAT.FastaIterator as FastaIterator

##---------------------------------------------------------------------------------------------
    
def main( argv = None ):

    parser = E.OptionParser( version = "%prog version: $Id: analyze_codonbias_shannon.py 2864 2010-03-03 10:18:16Z andreas $",
                                    usage = globals()["__doc__"] )

    parser.add_option( "-c", "--is-cds", dest="is_cds", action="store_true",
                       help = "input are cds sequences [%default]" )
    
    parser.set_defaults(
        is_cds = False,
        )
    
    (options, args) = E.Start( parser, argv = argv )

    options.stdout.write( "snpid\tidentifier\tpos\treference\tvariant\tcounts\tweight\n" )

    alphabet = "ACDEFGHIKLMNPQRSTVWY"
    
    snpid = 0

    for entry in FastaIterator.iterate( options.stdin ):
        identifier = entry.title

        if options.is_cds:
            cds_sequence = entry.sequence.upper()
            assert len(cds_sequence) % 3 == 0, \
                "length of sequence '%s' is not a multiple of 3" % entry.title

            sequence = Genomics.translate( cds_sequence )
            weights = []
            for pos, cds_pos in enumerate(range( 0, len(cds_sequence), 3)):
                codon = cds_sequence[cds_pos:cds_pos+3]
                counts = collections.defaultdict(int)
                for x in range(0,3):
                    rna = codon[x]
                    for na in "ACGT":
                        if na == rna: continue
                        taa = Genomics.translate(codon[:x] + na + codon[x+1:])
                        counts[taa] += 1
                weights.append( counts )

        else:
            sequence = entry.sequence.upper()
            counts = {}
            for x in alphabet: counts[x] = 1
            weights = [counts] * len(sequence)

        for pos, ref in enumerate( sequence ):

            if ref not in alphabet: continue
            w = weights[pos]
            t = float(sum(w.values()))
            for variant in alphabet:
                if variant == ref: continue
                snpid +=1
                options.stdout.write( 
                    "%s\n" % "\t".join(
                        ( "%010i" % snpid,
                          identifier,
                          str(pos+1),
                          ref, 
                          variant,
                          "%i" % w[variant],
                          "%6.4f" % (w[variant] / t),
                          )))
    
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
mali2summary.py - compute column stats for a multiple alignment
===============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a multiple alignment from stdin and outputs
a :term:`tsv` formatted table on stdout. Each row contains
summary information on a multiple alignment column.

Usage
-----

Example::

   python mali2table.py --help

Type::

   python mali2table.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Mali as Mali
import scipy
import scipy.stats

from CGAT.SequenceProperties import *

def main( argv = sys.argv ):

    parser = E.OptionParser( version = "%prog version: $Id$",
                                    usage = globals()["__doc__"])

    parser.add_option("-i", "--input-format", dest="input_format", type="choice",
                      choices=("plain", "fasta", "clustal", "stockholm" ),
                      help="input format of multiple alignment"  )
    
    parser.add_option("-a", "--alphabet", dest="alphabet", type="choice",
                      choices=("aa", "na"),
                      help="alphabet to use [default=%default].", )

    parser.add_option("-s", "--sections", dest="sections", type="choice", action="append",
                      choices = ("length", "composition", "entropy", "all"),
                      help="which sections to output" )

    parser.add_option( "-u", "--allow-duplicates", dest="allow_duplicates", action="store_true",
                      help="permit duplicate entries [default=%default]."  )

    parser.set_defaults(
        input_format="fasta",
        output_format="fasta",
        mask_chars = "nN",
        gap_chars = "-.",
        alphabet="na",
        sections = [],
        allow_duplicates = False,
        )

    (options, args) = E.Start( parser )

    if len(options.sections) == 0:
        raise ValueError("please supply at least one method." )

    if "all" in options.sections:
        options.sections = ["length", "composition", "entropy" ]

    counters = []

    def getCounter( section ):

        if options.alphabet == "na":
            if section == "length":
                s = SequencePropertiesLength()
            elif section == "composition":
                s = SequencePropertiesNA()
            elif section == "entropy":
                s = SequencePropertiesEntropy( "ACGT" )
            else:
                raise ValueError("unknown section %s" % section)
        elif options.alphabet == "aa":
            if section == "length":
                s = SequencePropertiesLength()
            elif section == "composition":
                s = SequencePropertiesAminoAcids()
            elif section == "entropy":
                s = SequencePropertiesEntropy( "ACDEFGHIKLMNPQRSTVWY" )
            else:
                raise ValueError("unknown section %s" % section)
        return s

    # read multiple alignment in various formats
    ## 1. read multiple alignment in various formats
    if options.allow_duplicates:
        mali = Mali.SequenceCollection()
    else:
        mali = Mali.Mali()

    mali.readFromFile( options.stdin, format = options.input_format )

    # do not use column, as it is a reserved word in sql
    options.stdout.write ("col" )
    for section in options.sections:
        options.stdout.write( "\t" + "\t".join(getCounter(section).getHeaders()))
    options.stdout.write( "\n" )

    columns = mali.getColumns()
    counter = E.Counter()

    for x, column in enumerate( columns ):
        counter.input += 1
        sequence = "".join(column)
        options.stdout.write( "%i" % x )

        for section in options.sections:
            s = getCounter( section )
            s.loadSequence( sequence )
            options.stdout.write( "\t" + "\t".join(s.getFields()) )

        options.stdout.write("\n")
        counter.output += 1

    E.info( "%s" % str(counter ) )
        
    E.Stop()
    
##------------------------------------------------------------
if __name__ == '__main__':
    sys.exit( main( sys.argv ) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
fastq2solid.py - convert fastq to solid
=======================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Converts a fastq-formatted file to two solid
output files (.csfasta and .qual)

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import math
import random

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Fastq as Fastq

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-f", "--change-format", dest="change_format", type="choice",
                      choices = ('sanger', 'solexa', 'phred64', 'integer'  ),
                      help="guess quality score format and set quality scores to format [default=%default]."  )

    parser.add_option( "--guess-format", dest="guess_format", type="choice",
                      choices = ('sanger', 'solexa', 'phred64', 'integer' ),
                      help="quality score format to assume if ambiguous [default=%default]."  )

    parser.add_option( "--pattern", dest="pattern", type="string",
                       help="filename prefix [default=%default]."  )

    parser.set_defaults(
        change_format = None,
        guess_format = None,
        pattern = "%s.gz"
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    c = E.Counter()

    outfile_seq = IOTools.openFile( options.pattern % "csfasta", "w" )
    outfile_qual = IOTools.openFile( options.pattern % "qual", "w" )
    
    if options.change_format:
        iter = Fastq.iterate_convert( options.stdin, 
                                         format = options.change_format,
                                         guess = options.guess_format )
    else:
        iter = Fastq.iterate( options.stdin )

    for record in iter:
        c.input += 1
        outfile_seq.write(">%s\n%s\n" % (record.identifier, record.seq ))
        outfile_qual.write(">%s\n%s\n" % (record.identifier, record.quals ))
        c.output += 1

    outfile_seq.close()
    outfile_qual.close()

    ## write footer and output benchmark information.
    E.info( "%s" % str(c) )
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: chain2psl.py 2899 2010-04-13 14:37:37Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
chain2psl.py - convert a chain file to a psl file
=================================================

:Author: Andreas Heger
:Release: $Id: chain2psl.py 2899 2010-04-13 14:37:37Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

compare two `chain <http://www.breyer.com/ucsc/htdocs/goldenPath/help/chain.html>`_ 
formatted files.

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse
import collections

import CGAT.IOTools as IOTools
import CGAT.Experiment as E
import alignlib

def chain_iterator( infile ):
    lines = []
    for line in infile:

        if line.startswith("#"): continue
        if line.strip() == "": continue
        if line.startswith("chain"):
            if lines: yield lines
            lines = []
        lines.append( line )

    yield lines

def validateChain( infile ):
    '''validate a chain file.

    No overlapping target coordinates.
    '''
    
    pairs_t2q = collections.defaultdict( alignlib.makeAlignmentBlocks )
    pairs_q2t = collections.defaultdict( alignlib.makeAlignmentBlocks )

    
    for lines in chain_iterator( infile ):
        
        ( _, 
          _, 
          target_contig,
          target_length,
          target_strand,
          target_start,
          target_end,
          query_contig,
          query_length,
          query_strand,
          query_start, 
          query_end,
          alignment_id ) = lines[0][:-1].split()

        (query_start, 
         query_end,
         query_length,
         target_start, 
         target_end,
         target_length) = \
         [ int(x) for x in 
           (query_start, 
            query_end,
            query_length,
            target_start, 
            target_end,
            target_length) ]
         
        # target_strand is always positive
        assert( target_strand == "+" )
        
        map_target2query = pairs_t2q[ target_contig ]
        map_query2target = pairs_q2t[ query_contig ]

        qstart, tstart = query_start, target_start
        
        for line in lines[1:-1]:
            size, dt, dq = [int(x) for x in line[:-1].split() ]
            map_target2query.addDiagonal( tstart,
                                          tstart + size,
                                          0 )
            map_query2target.addDiagonal( qstart,
                                          qstart + size, 
                                          0 )

            qstart += size + dq
            tstart += size + dt

        size = int(lines[-1][:-1])
        map_target2query.addDiagonal( tstart,
                                      tstart + size,
                                      0 )
        map_query2target.addDiagonal( qstart,
                                      qstart + size,
                                      0 )

    try: 
        x = map_query2target.mapRowToCol(0)
    except RuntimeError:
        E.info( "query is not unique - this is ok." )

    try:
        x = map_target2query.mapRowToCol(0)
    except RuntimeError:
        E.info( "target is not unique" )
        return False
    
    return True

def buildPairs( infile ):
    '''read a chain file.

    build target2query alignments.
    The target is always on the positive strand.
    '''
    pairs = collections.defaultdict( alignlib.makeAlignmentBlocks )

    def chain_iterator( infile ):
        lines = []
        for line in infile:
            
            if line.startswith("#"): continue
            if line.strip() == "": continue
            if line.startswith("chain"):
                if lines: yield lines
                lines = []
            lines.append( line )
            
        yield lines

    for lines in chain_iterator( infile ):
        
        ( _, 
          _, 
          target_contig,
          target_length,
          target_strand,
          target_start,
          target_end,
          query_contig,
          query_length,
          query_strand,
          query_start, 
          query_end,
          alignment_id ) = lines[0][:-1].split()

        (query_start, 
         query_end,
         query_length,
         target_start, 
         target_end,
         target_length) = \
         [ int(x) for x in 
           (query_start, 
            query_end,
            query_length,
            target_start, 
            target_end,
            target_length) ]
         
        # target_strand is always positive
        assert( target_strand == "+" )
        
        map_target2query = pairs[ (target_contig, query_contig, query_strand ) ]

        qstart, tstart = query_start, target_start
        
        for line in lines[1:-1]:
            size, dt, dq = [int(x) for x in line[:-1].split() ]
            map_target2query.addDiagonal( tstart,
                                          tstart + size,
                                          qstart - tstart )
            qstart += size + dq
            tstart += size + dt

        size = int(lines[-1][:-1])

        map_target2query.addDiagonal( tstart,
                                      tstart + size,
                                      qstart - tstart )


    return pairs

DiffResult = collections.namedtuple( "DiffResult","total same different unique")
def compareChains( pairs1, pairs2 ):
    '''compare chains in pairs1 versus those in pairs2'''
    

    result = {}
    for key1, chain1 in pairs1.iteritems():
        E.debug( "comparing %s" % str(key1) )

        ntotal = chain1.getNumAligned()

        if key1 not in pairs2:
            result[ key1 ] = DiffResult._make( (ntotal, 0, 0, ntotal) )
            continue

        chain2 = pairs2[key1]
        nsame = alignlib.getAlignmentIdentity( chain1, chain2, alignlib.RR )
        noverlap = alignlib.getAlignmentOverlap( chain1, chain2, alignlib.RR )
        ndifferent = noverlap - nsame
        nunique = ntotal - noverlap

        result[ key1 ] = DiffResult._make( (ntotal, nsame, ndifferent, nunique ) )

    return result

def outputMismatches( pairs1, pairs2, 
                      output_mismatches = False, 
                      output_unique = False,
                      output_matches = False ):
    '''output mismatches.

    This is a very slow operation.
    '''

    outfile = sys.stdout
    

    for key1, chain1 in pairs1.iteritems():
        E.debug( "comparing %s" % str(key1) )

        if key1 not in pairs2:
            outfile.write( "%s\t%s\t%s\t%i\t%i\t-1\n" %
                           ( key1 + (chain1.getRowFrom(), chain2.getRowTo()) ) )
            continue
        
        chain2 = pairs2[key1]
        reg_start = chain1.getRowFrom()
        for pos in xrange(chain1.getRowFrom(), chain2.getRowTo()):
            x = chain1.mapRowToCol( pos )
            y = chain2.mapRowToCol( pos )

            if x == y: 
                if output_matches:
                    outfile.write( "%s\t%s\t%s\t%i\t%i\t%i\t%s\n" %
                                   ( key1 + (pos, x, y, "=") ) )
                continue

            mismatch =  x != -1 and y != -1
            
            if mismatch:
                if output_mismatches:
                    outfile.write( "%s\t%s\t%s\t%i\t%i\t%i\t%i\n" %
                                   ( key1 + (pos, x, y, x-y) ) )
            else:
                if output_unique:
                    outfile.write( "%s\t%s\t%s\t%i\t%i\t%i\t%s\n" %
                                   ( key1 + (pos, x, y, "-") ) )
                            

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: chain2psl.py 2899 2010-04-13 14:37:37Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-m", "--output-mismatches", dest="output_mismatches", action = "store_true",
                       help = "output mismatches [%default]" )

    parser.add_option( "-a", "--output-matches", dest="output_matches", action = "store_true",
                       help = "output matches [%default]" )

    parser.add_option( "-u", "--output-unique", dest="output_unique", action = "store_true",
                       help = "output unique positions [%default]" )

    parser.add_option( "-r", "--restrict", dest="restrict", type = "string",
                       help = "restrict analysis to a chromosome pair (chr1:chr1:+) [%default]" )

    parser.set_defaults(
        output_mismatches = False,
        output_unique = False,
        restrict = None 
        )


    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) != 2:
        raise ValueError( "expected two chain files" )

    filename_chain1, filename_chain2 = args

    E.info( "validating chain 1")
    if not validateChain( IOTools.openFile( filename_chain1 ) ):
        E.warn( "validation failed - exiting" )
        return 1
        
    E.info( "validating chain 2")
    if not validateChain( IOTools.openFile( filename_chain2 ) ):
        E.warn( "validation failed - exiting" )
        return 1

    E.info( "building pairs for %s" % filename_chain1 )
    pairs1 = buildPairs( IOTools.openFile( filename_chain1 ) )
    E.info( "read %i pairs" % len(pairs1) )

    E.info( "building pairs for %s" % filename_chain2 )
    pairs2 = buildPairs( IOTools.openFile( filename_chain2 ) )
    E.info( "read %i pairs" % len(pairs2) )

    if options.restrict: 
        restrict = tuple(options.restrict.split(":"))
        pairs1 = { restrict: pairs1[restrict] }
        pairs2 = { restrict: pairs2[restrict] }

    E.info( "comparing 1 -> 2")
    comparison1 = compareChains( pairs1, pairs2 )
    E.info( "comparing 2 -> 1")
    comparison2 = compareChains( pairs2, pairs1 )

    all_keys = sorted(list( set(comparison1.keys() + comparison2.keys())))
    
    outfile = options.stdout
    headers = ("mapped", "identical", "different", "unique")
    outfile.write( "contig1\tcontig2\tstrand\t%s\t%s\t%s\t%s\n" %\
                       ( 
            "\t".join( ["%s1" % x for x in headers ] ),
            "\t".join( ["p%s1" % x for x in headers ] ),
            "\t".join( ["%s2" % x for x in headers ] ),
            "\t".join( ["p%s2" % x for x in headers ] )))
                         
    totals = E.Counter()

    for key in all_keys:
        outfile.write( "%s\t%s\t%s" % key )
        
        if key in comparison1:
            c = comparison1[key]
            outfile.write( "\t%i\t%i\t%i\t%i\t" % (c.total, c.same, c.different, c.unique ) )
            outfile.write( "\t".join( [ IOTools.prettyPercent( x, c.total ) for x in c ] ) )

            totals.total1 += c.total
            totals.same1 += c.same
            totals.different1 += c.different
            totals.unique1 += c.unique
        else:
            outfile.write( "\t%i\t%i\t%i\t%i\t" % (0,0,0,0) )
            outfile.write( "\t%i\t%i\t%i\t%i" % (0,0,0,0) )

        if key in comparison2:
            c = comparison2[key] 
            outfile.write( "\t%i\t%i\t%i\t%i\t" % (c.total, c.same, c.different, c.unique ) )
            outfile.write( "\t".join( [ IOTools.prettyPercent( x, c.total ) for x in c ] ) )

            totals.same2 += c.same
            totals.total2 += c.total
            totals.different2 += c.different
            totals.unique2 += c.unique 
        else:
            outfile.write( "\t%i\t%i\t%i\t%i\t" % (0,0,0,0) )
            outfile.write( "\t%i\t%i\t%i\t%i" % (0,0,0,0) )

        outfile.write("\n")

    outfile.write( "total\ttotal\t.\t" )
    outfile.write( "\t".join( map(str, ( totals.total1,
                                         totals.same1, 
                                         totals.different1,
                                         totals.unique1,
                                         IOTools.prettyPercent( totals.total1, totals.total1 ),
                                         IOTools.prettyPercent( totals.same1, totals.total1 ),
                                         IOTools.prettyPercent( totals.different1, totals.total1 ),
                                         IOTools.prettyPercent( totals.unique1, totals.total1 ),
                                         totals.total2,
                                         totals.same2, 
                                         totals.different2,
                                         totals.unique2,
                                         IOTools.prettyPercent( totals.total2, totals.total2 ),
                                         IOTools.prettyPercent( totals.same2, totals.total2 ),
                                         IOTools.prettyPercent( totals.different2, totals.total2 ),
                                         IOTools.prettyPercent( totals.unique2, totals.total2 ),
                                         ) ) ) + "\n" )
                                        
    
    # output mismapped residues
    if options.output_mismatches or options.output_unique:
        outputMismatches( pairs1, pairs2,
                          output_mismatches = options.output_mismatches,
                          output_unique = options.output_unique,
                          output_matches = options.output_matches,
                          )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
fasta2nj.py - convert fasta file to nj input
============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

convert a fasta file to NJ input.

This script translates identifiers like

species|transcripts|gene|class to transcript_species GENEID=gene

Usage
-----

Example::

   python fasta2nj.py --help

Type::

   python fasta2nj.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import tempfile
import time
import optparse
import math
import glob

import CGAT.Experiment as E
import CGAT.IOTools as IOTools

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: fasta2nj.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option( "-m", "--map", dest="filename_map", type="string",
                       help="filename with mapping of species ids to swissprot species ids." )
    
    parser.set_defaults(
        separator="|",
        filename_map = None,
        )

    (options, args) = E.Start( parser )

    if options.filename_map:
        map_species2sp = IOTools.ReadMap( open(options.filename_map, "r"))

    ninput, noutput, nerrors = 0, 0, 0
    for line in sys.stdin:
        if line[0] == ">":
            ninput += 1

            id = re.match(">([^/ \t]+)", line[:-1]).groups()[0]
            data = id.split( options.separator )

            species = data[0]
            
            if len(data) == 2:
                gene = data[1]
                transcript = None
            elif len(data) >= 3:
                gene = data[2]
                transcript = data[1]
            
            if map_species2sp:
                try:
                    species = map_species2sp[species]
                except IndexError:
                    nerrors += 1
                    if options.loglevel >= 1:
                        options.stdlog.write("# could not map species %s\n" % species)
            if transcript:
                options.stdout.write( ">%s_%s GENEID=%s\n" % ( transcript, species, gene ) )
            else:
                options.stdout.write( ">%s_%s\n" % ( species, gene ) )                
            noutput += 1
        else:
            options.stdout.write( line )

    if options.loglevel >= 1:
        options.stdlog.write("# ninput=%i, noutput=%i, nerrors=%i\n" % (ninput, noutput, nerrors))
    E.Stop()
